
from vsg.token import block_specification as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import generate_specification


def classify(iToken, lObjects):
    '''
    block_specification ::=
        architecture_name
      | block_statement_label
      | generate_statement_label [ ( generate_specification ) ]
    '''

    if utils.find_in_next_n_tokens('(', 2, iToken, lObjects):
        iCurrent = utils.assign_next_token(token.generate_statement_label, iToken, lObjects)
        iCurrent = utils.assign_next_token_required('(', token.open_parenthesis, iCurrent, lObjects)
        iCurrent = generate_specification.classify(iCurrent, lObjects)
        iCurrent = utils.assign_next_token_required(')', token.close_parenthesis, iCurrent, lObjects)
    else:
        iCurrent = utils.assign_next_token(token.architecture_name, iToken, lObjects)

    return iCurrent




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.process_statement.begin_keyword)


class rule_003(token_indent):
    '''
    This rule checks the indent of the **begin** keyword.

    **Violation**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full
                        ) is
         begin

    **Fix**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full
                        ) is
       begin
    '''

    def __init__(self):
        token_indent.__init__(self, 'process', '003', lTokens)




from vsg.rules import previous_line as Rule

from vsg.token import record_type_definition as token


class rule_201(Rule):
    '''
    This rule checks for blank lines above the **end** keyword.

    |configuring_previous_line_rules_link|

    **Violation**

    .. code-block:: vhdl

       type t_record is record
         a : std_logic;
         b : std_logic;

       end record    t_record;

    **Fix**

    .. code-block:: vhdl

       type t_record is record
         a : std_logic;
         b : std_logic;
       end record t_record;
    '''

    def __init__(self):
        Rule.__init__(self, 'record_type_definition', '201', [token.end_keyword])
        self.style = 'no_blank_line'




from vsg.rules import blank_lines_between_token_pairs

from vsg import token

lTokenPairs = []
lTokenPairs.append([token.concurrent_simple_signal_assignment.target, token.concurrent_simple_signal_assignment.semicolon])
lTokenPairs.append([token.concurrent_conditional_signal_assignment.target, token.concurrent_conditional_signal_assignment.semicolon])
lTokenPairs.append([token.concurrent_selected_signal_assignment.with_keyword, token.concurrent_selected_signal_assignment.semicolon])


class rule_010(blank_lines_between_token_pairs):
    '''
    This rule removes blank lines within concurrent signal assignments.

    **Violation**

    .. code-block:: vhdl

       wr_en <= '0' when q_wr_en = '1' else

            '1';

       w_foo <= I_FOO when ((I_BAR = '1') and

                            (I_CRUFT = '1')) else

                '0';

    **Fix**

    .. code-block:: vhdl

       wr_en <= '0' when q_wr_en = '1' else
                '1';

       w_foo <= I_FOO when ((I_BAR = '1') and
                            (I_CRUFT = '1')) else
                '0';
    '''

    def __init__(self):
        blank_lines_between_token_pairs.__init__(self, 'concurrent', '010', lTokenPairs)
        self.configuration_documentation_link = None




from vsg.rules.whitespace_between_tokens import Rule

from vsg.token import architecture_body as token


class rule_022(Rule):
    '''
    This rule checks for a single space before the entity name in the end architecture declaration.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       end architecture    fifo;

    **Fix**

    .. code-block:: vhdl

       end architecture fifo;
    '''
    def __init__(self):
        Rule.__init__(self, 'architecture', '022')
        self.left_token = token.end_architecture_keyword
        self.right_token = token.architecture_simple_name




from vsg import violation

from vsg.vhdlFile import utils
from vsg.rule_group import case


class consistent_case_of_tokens_from_between_tokens_applied_to_region(case.Rule):
    '''
    Checks the case for words.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens: list of token type objects
       token type to apply the case check against
    '''

    def __init__(self, name, identifier, lTokens, oStart, oEnd, oRegionStart, oRegionEnd, lIgnore=None):
        case.Rule.__init__(self, name=name, identifier=identifier)
        self.subphase = 2
        self.lTokens = lTokens
        self.oStart = oStart
        self.oEnd = oEnd
        self.oRegionStart = oRegionStart
        self.oRegionEnd = oRegionEnd
        if lIgnore is None:
            self.lIgnoreTokens = []
        else:
            self.lIgnoreTokens = lIgnore
        self.configuration_documentation_link = None

    def analyze(self, oFile):
        self._print_debug_message('Analyzing rule: ' + self.unique_id)

        lTargetTypes = oFile.get_tokens_matching_in_range_bounded_by_tokens(self.lTokens, self.oStart, self.oEnd)
        lTargetValues = []
        lTargetValuesLower = []
        for oTargetType in lTargetTypes:
            oToken = oTargetType.get_tokens()[0]
            lTargetValues.append(oToken.get_value())
            lTargetValuesLower.append(oToken.get_value().lower())

        lToi = oFile.get_tokens_bounded_by(self.oRegionStart, self.oRegionEnd)

        for oToi in lToi:
            iLine, lTokens = utils.get_toi_parameters(oToi)

            for iToken, oToken in enumerate(lTokens):

                iLine = utils.increment_line_number(iLine, oToken)

                if is_token_in_ignore_token_list(oToken, self.lIgnoreTokens):
                    continue

                sTokenValue = oToken.get_value()
                sTokenValueLower = sTokenValue.lower()

                for sTargetValue, sTargetValueLower in zip(lTargetValues, lTargetValuesLower):
                    if sTokenValueLower == sTargetValueLower:
                        if sTokenValue != sTargetValue:
                            sSolution = 'Change "' + sTokenValue + '" to "' + sTargetValue + '"'
                            oNewToi = oToi.extract_tokens(iToken, iToken)
                            oViolation = violation.New(iLine, oNewToi, sSolution)
                            dAction = {}
                            dAction['constant'] = sTargetValue
                            dAction['found'] = sTokenValue
                            oViolation.set_action(dAction)
                            self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dActions = oViolation.get_action()
        lTokens[0].set_value(dActions['constant'])
        oViolation.set_tokens(lTokens)


def is_token_in_ignore_token_list(oToken, lIgnoreTokens):
    for oIgnoreToken in lIgnoreTokens:
        if isinstance(oToken, oIgnoreToken):
            return True
    return False




from vsg import parser

from vsg.vhdlFile.extract import tokens


def is_token_at_start_of_line(iIndex, oTokenMap):
    if oTokenMap.is_token_at_index(parser.carriage_return, iIndex - 1):
        return True
    if oTokenMap.is_token_at_index(parser.carriage_return, iIndex - 2) and is_token_at_index_whitespace(oTokenMap, iIndex - 1):
        return True
    return False


def is_token_at_end_of_line(iIndex, oTokenMap):
    if oTokenMap.is_token_at_index(parser.carriage_return, iIndex + 1):
        return True
    if oTokenMap.is_token_at_index(parser.comment, iIndex + 1):
        return True
    if oTokenMap.is_token_at_index(parser.whitespace, iIndex + 1):
        if oTokenMap.is_token_at_index(parser.carriage_return, iIndex + 2):
            return True
        if oTokenMap.is_token_at_index(parser.comment, iIndex + 2):
            return True
    return False


def get_indexes_of_token_list(lTokens, oTokenMap):
    lReturn = []
    for oToken in lTokens:
        lReturn.extend(oTokenMap.get_token_indexes(oToken))

    lReturn.sort()

    return lReturn


def get_indexes_of_token_pairs(lPairs, oTokenMap):
    lReturn = []
    for lPair in lPairs:
        lStart, lEnd = oTokenMap.get_token_pair_indexes(lPair[0], lPair[1])
        for iStart, iEnd in zip(lStart, lEnd):
            lReturn.append([iStart, iEnd])

    return lReturn


def get_line_numbers_of_indexes_in_list(lIndexes, oTokenMap):
    lReturn = []
    for iIndex in lIndexes:
        lReturn.append(oTokenMap.get_line_number_of_index(iIndex))

    lReturn.sort()

    return lReturn


def is_index_between_indexes(iIndex, lStart, lEnd, bInclusive=False):
    for iStart, iEnd in zip(lStart, lEnd):
        if bInclusive:
            if iStart <= iIndex and iIndex <= iEnd:
                return True
        else:
            if iStart < iIndex and iIndex < iEnd:
                return True
    return False


def filter_tokens_which_start_a_line(oToken, oTokenMap):
    lIndexes = oTokenMap.get_token_indexes(oToken)
    return filter_indexes_which_start_a_line(lIndexes, oTokenMap)


def filter_indexes_which_start_a_line(lIndexes, oTokenMap):
    lReturn = []
    for iIndex in lIndexes:
        if is_token_at_start_of_line(iIndex, oTokenMap):
            lReturn.append(iIndex)
    return lReturn


def filter_tokens_between_tokens(lTokens, oStart, oEnd, oTokenMap):
    lStart = oTokenMap.get_token_indexes(oStart)
    lEnd = oTokenMap.get_token_indexes(oEnd)

    lReturn = []
    for oToken in lTokens:
        for iStart, iEnd in zip(lStart, lEnd):
            lReturn.extend(oTokenMap.get_token_indexes_between_indexes(oToken, iStart, iEnd))
    return lReturn


def filter_tokens_between_tokens_unless_token_exists_between_them(lTokens, oStart, oEnd, oStop, oTokenMap):
    lStart = oTokenMap.get_token_indexes(oStart)
    lEnd = oTokenMap.get_token_indexes(oEnd)
    lStop = oTokenMap.get_token_indexes(oStop)
    lReturn = []
    for oToken in lTokens:
        for iStart in lStart:
            for iEnd in lEnd:
                if iEnd > iStart:
                    for iStop in lStop:
                        if iStart < iStop < iEnd:
                            break
                    else:
                        lReturn.extend(oTokenMap.get_token_indexes_between_indexes(oToken, iStart, iEnd))
                        break
    return lReturn


def get_indexes_of_tokens_between(lStartToken, lEndTokens, oTokenMap):
    '''
    This function will take a list of start tokens and a list of end tokens.
    It will return a list of indexes of either a start token or an end token.
    A start token index will be returned if another start token was found before an end token.
    An end token index will be returned if the next token index is a start token.
    '''
    lReturn = []
    lStartIndexes = oTokenMap.get_token_indexes(lStartToken)
    lEndIndexes = get_indexes_of_token_list(lEndTokens, oTokenMap)

    for iStartIndex, iStart in enumerate(lStartIndexes):
        try:
            iNextStart = lStartIndexes[iStartIndex + 1]
        except IndexError:
            try:
                iNextStart = lEndIndexes[-1] + 1
            except IndexError:
                lReturn.append(iStartIndex)
                return lReturn
        iEndIndex = iStart
        for iEnd in lEndIndexes:
            if iEnd > iStart and iEnd < iNextStart:
                iEndIndex = iEnd
            if iEnd > iNextStart:
                break
        lReturn.append(iEndIndex)
    return lReturn


def get_all_blank_lines_above_indexes(lIndexes, lAllTokens, oTokenMap):
    lReturn = []
    for iIndex in lIndexes:
        iLine = oTokenMap.get_line_number_of_index(iIndex)
        iEnd = oTokenMap.get_index_of_carriage_return_before_index(iIndex)
        iStart = oTokenMap.get_index_of_previous_non_whitespace_token_before_index(iEnd)
        if iStart is None:
            continue
        iStart = oTokenMap.get_index_of_carriage_return_after_index(iStart)
        if iStart != iEnd:
            lReturn.append(tokens.New(iStart, iLine, lAllTokens[iStart:iEnd]))

    return lReturn


def is_token_at_index_whitespace(oTokenMap, iIndex):
    if oTokenMap.is_token_at_index(parser.whitespace, iIndex):
        return True
    return False


def is_token_at_index_carriage_return(oTokenMap, iIndex):
    if oTokenMap.is_token_at_index(parser.carriage_return, iIndex):
        return True
    return False




from vsg.rules import move_token_next_to_another_token_if_it_exists_between_tokens

from vsg.token import context_declaration as token

lBetweenTokens = [token.end_keyword, token.semicolon]


class rule_009(move_token_next_to_another_token_if_it_exists_between_tokens):
    '''
    This rule checks the **context** keyword is on the same line as the end context keyword.

    **Violation**

    .. code-block:: vhdl

       end
       context c1;

    **Fix**

    .. code-block:: vhdl

       end context
         c1;
    '''

    def __init__(self):
        move_token_next_to_another_token_if_it_exists_between_tokens.__init__(self, 'context', '009', token.end_keyword, token.end_context_keyword, lBetweenTokens)
        self.subphase = 1
        self.solution = 'Move identifier next to *end* keyword'




from vsg import token

from vsg.rules import spaces_before_and_after_tokens_when_bounded_by_tokens as Rule

lTokens = []
lTokens.append(token.mode.out_keyword)

lBetween = [token.port_clause.open_parenthesis, token.port_clause.close_parenthesis]


class rule_008(Rule):
    '''
    This rule checks for spaces before and after the **out** mode keyword.

    |configuring_port_mode_alignment_link|

    **Violation**

    .. code-block:: vhdl

       port (
         WR_EN    : in    std_logic;
         RD_EN    : in    std_logic;
         OVERFLOW : out   std_logic
       );

    **Fix**

    .. code-block:: vhdl

       port (
         WR_EN    : in    std_logic;
         RD_EN    : in    std_logic;
         OVERFLOW : out std_logic
       );
    '''
    def __init__(self):
        Rule.__init__(self, 'port', '008', lTokens, lBetween)
        self.spaces_before = 1
        self.spaces_after = 3
        self.configuration_documentation_link = 'configuring_port_mode_alignment_link'




from vsg import parser


class to(parser.keyword):
    '''
    unique_id = direction : to
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class downto(parser.keyword):
    '''
    unique_id = direction : downto
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)




from vsg import token

from vsg.rules.whitespace_before_token import Rule

lTokens = []
lTokens.append(token.constant_declaration.assignment_operator)


class rule_010(Rule):
    '''
    This rule checks for a single space before the := keyword in constant declarations.
    Having a space makes it clearer where the assignment occurs on the line.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       constant size : integer:= 1;
       constant width : integer   := 10;

    **Fix**

    .. code-block:: vhdl

       constant size : integer := 1;
       constant width : integer := 10;
    '''
    def __init__(self):
        Rule.__init__(self, 'constant', '010', lTokens)
        self.number_of_spaces = '>=1'




from vsg.token import constrained_array_definition as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import index_constraint
from vsg.vhdlFile.classify import subtype_indication


def detect(iToken, lObjects):
    '''
    constrained_array_definition ::=
        array index_constraint of *element*_subtype_indication
    '''

    if utils.is_next_token('array', iToken, lObjects):
        if not utils.find_in_next_n_tokens('<>', 5, iToken, lObjects):
            return classify(iToken, lObjects)
        else:
            return iToken

    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('array', token.array_keyword, iToken, lObjects)

    iCurrent = index_constraint.classify(iToken, lObjects)

    iCurrent = utils.assign_next_token_required('of', token.of_keyword, iCurrent, lObjects)

    iCurrent = subtype_indication.classify(iCurrent, lObjects)

    return iCurrent




from vsg import parser


class in_keyword(parser.keyword):
    '''
    unique_id = force_mode : in_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class out_keyword(parser.keyword):
    '''
    unique_id = force_mode : out_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)




from vsg import violation

from vsg.rule_group import naming


class token_suffix_between_tokens(naming.Rule):
    '''
    Checks the suffix of a token.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : list of parser object types
       object types to check the prefix

    lSuffixes: string list
       acceptable suffixes
    '''

    def __init__(self, name, identifier, lTokens, oStart, oEnd):
        naming.Rule.__init__(self, name=name, identifier=identifier)
        self.lTokens = lTokens
        self.suffixes = None
        self.configuration.append('suffixes')
        self.fixable = False
        self.disable = True
        self.oStart = oStart
        self.oEnd = oEnd

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_matching_in_range_bounded_by_tokens(self.lTokens, self.oStart, self.oEnd)

    def _analyze(self, lToi):
        lSuffixLower = []
        for sSuffix in self.suffixes:
            lSuffixLower.append(sSuffix.lower())
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            sToken = lTokens[0].get_value().lower()
            bValid = False
            for sSuffix in lSuffixLower:
                if sToken.endswith(sSuffix):
                    bValid = True
            if not bValid:
                sSolution = 'Suffix ' + lTokens[0].get_value() + ' with one of the following: ' + ', '.join(self.suffixes)
                oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
                self.add_violation(oViolation)





from vsg import parser


def classify(lTokens, lObjects):
    '''
    Classifies preprocessor commands

    '''
    # Check for entity
    try:
        if lTokens[0].startswith('#') or (lTokens[0].startswith(' ') and lTokens[1].startswith('#')):
            lObjects.clear()
            lObjects.append(parser.preprocessor(''.join(lTokens)))
    except IndexError:
        return




from vsg import token

from vsg.rules.whitespace_before_token import Rule

lTokens = []
lTokens.append(token.simple_variable_assignment.assignment)
lTokens.append(token.conditional_variable_assignment.assignment)
lTokens.append(token.selected_variable_assignment.assignment)


class rule_003(Rule):
    '''
    This rule checks for at least a single space before the assignment.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

         counter:= 0;
         count := counter + 1;

    **Fix**

    .. code-block:: vhdl

         counter := 0;
         count := counter + 1;
    '''
    def __init__(self):
        Rule.__init__(self, 'variable_assignment', '003', lTokens)
        self.number_of_spaces = '>=1'




from vsg import parser
from vsg import token

from vsg.rules import consistent_token_case

lTokens = []
lTokens.append(token.signal_declaration.identifier)

lIgnore = []
lIgnore.append(token.interface_signal_declaration.identifier)
lIgnore.append(token.interface_unknown_declaration.identifier)
lIgnore.append(token.interface_constant_declaration.identifier)
lIgnore.append(token.interface_variable_declaration.identifier)
lIgnore.append(token.association_element.formal_part)
lIgnore.append(token.entity_declaration.identifier)
lIgnore.append(token.entity_declaration.entity_simple_name)
lIgnore.append(token.architecture_body.entity_name)
lIgnore.append(parser.whitespace)
lIgnore.append(parser.carriage_return)
lIgnore.append(parser.blank_line)


class rule_014(consistent_token_case):
    '''
    This rule checks for consistent capitalization of signal names.

    **Violation**

    .. code-block:: vhdl

       architecture rtl of entity1 is

         signal sig1 : std_logic;
         signal sig2 : std_logic;

       begin

         proc_name : process (siG2) is
         begin

           siG1 <= '0';

           if (SIG2 = '0') then
             sIg1 <= '1';
           elisif (SiG2 = '1') then
             SIg1 <= '0';
           end if;

         end process proc_name;

       end architecture rtl;

    **Fix**

    .. code-block:: vhdl

       architecture rtl of entity1 is

         signal sig1 : std_logic;
         signal sig2 : std_logic;

         proc_name : process (sig2) is
         begin

           sig1 <= '0';

           if (sig2 = '0') then
             sig1 <= '1';
           elisif (sig2 = '1') then
             sig1 <= '0';
           end if;

         end process proc_name;

       end architecture rtl;
    '''

    def __init__(self):
        consistent_token_case.__init__(self, 'signal', '014', lTokens, lIgnore)




from vsg.vhdlFile.classify import constrained_array_definition
from vsg.vhdlFile.classify import unbounded_array_definition


def detect(iToken, lObjects):
    '''
    array_type_definition ::=
        unbounded_array_definition
      | constrained_array_definition
    '''

    iCurrent = unbounded_array_definition.detect(iToken, lObjects)
    if iCurrent != iToken:
        return iCurrent

    iCurrent = constrained_array_definition.detect(iToken, lObjects)
    if iCurrent != iToken:
        return iCurrent

    return iToken





from vsg.rules import move_token_right_to_next_non_whitespace_token as Rule

from vsg import token

lTokens = []
lTokens.append(token.process_statement.label_colon)


class rule_038(Rule):
    '''
    This rule checks a label colon is on the same line as the **process** or **postponed** keyword.

    **Violation**

    .. code-block:: vhdl

       label :
       process

    **Fix**

    .. code-block:: vhdl

       label
       : process
    '''

    def __init__(self):
        Rule.__init__(self, 'process', '038', lTokens)
        self.subphase = 2




from vsg.rules import insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment

from vsg.token import generic_clause as token

lTokens = []
lTokens.append(token.open_parenthesis)


class rule_013(insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment):
    '''
    This rule checks for the **generic** keyword on the same line as a generic declaration.

    **Violation**

    .. code-block:: vhdl

       generic (g_depth : integer := 512;

    **Fix**

    .. code-block:: vhdl

       generic (
         g_depth : integer := 512;
    '''

    def __init__(self):
        insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment.__init__(self, 'generic', '013', lTokens)
        self.solution = 'Move generic parameter to the next line.'




from vsg.token import package_declaration as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import package_declarative_part
from vsg.vhdlFile.classify import package_header


def detect(iToken, lObjects):
    '''
    package_declaration ::=
        package identifier is
            package_header
            package_declarative_part
        end [ package ] [ package_simple_name ] ;
    '''

    iCurrent = utils.find_next_token(iToken, lObjects)
    if utils.object_value_is(lObjects, iCurrent, 'package'):
        if not utils.find_in_next_n_tokens('body', 5, iCurrent, lObjects):
            if not utils.find_in_next_n_tokens('new', 5, iCurrent, lObjects):
                return classify(iToken, lObjects)
        else:
            return iToken

    return iToken


def classify(iToken, lObjects):

    iCurrent = classify_opening_declaration(iToken, lObjects)

    iCurrent = package_header.detect(iCurrent, lObjects)

    iCurrent = package_declarative_part.detect(iCurrent, lObjects)

    iCurrent = classify_closing_declaration(iCurrent, lObjects)

    return iCurrent


def classify_opening_declaration(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('package', token.package_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token(token.identifier, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('is', token.is_keyword, iCurrent, lObjects)

    return iCurrent


def classify_closing_declaration(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('end', token.end_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token_if('package', token.end_package_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token_if_not(';', token.end_package_simple_name, iToken, lObjects)
    iCurrent = utils.assign_next_token_required(';', token.semicolon, iToken, lObjects)

    return iCurrent




from vsg.token import package_header as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import generic_clause
from vsg.vhdlFile.classify import generic_map_aspect


def detect(iToken, lObjects):
    '''
    package_header ::=
        [ generic_clause
        [ generic_map_aspect ; ] ]
    '''

    iCurrent = generic_clause.detect(iToken, lObjects)

    iLast = iCurrent
    iCurrent = generic_map_aspect.detect(iCurrent, lObjects)
    if iCurrent != iLast:
        return utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iToken




from vsg import parser


class procedure_keyword(parser.keyword):
    '''
    unique_id = procedure_specification : procedure_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class designator(parser.designator):
    '''
    unique_id = procedure_specification : designator
    '''

    def __init__(self, sString):
        parser.designator.__init__(self, sString)


class parameter_keyword(parser.keyword):
    '''
    unique_id = procedure_specification : parameter_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class open_parenthesis(parser.open_parenthesis):
    '''
    unique_id = procedure_specification : open_parenthesis
    '''

    def __init__(self, sString='('):
        parser.open_parenthesis.__init__(self)


class close_parenthesis(parser.close_parenthesis):
    '''
    unique_id = procedure_specification : close_parenthesis
    '''

    def __init__(self, sString=')'):
        parser.close_parenthesis.__init__(self)




from vsg import deprecated_rule


class rule_024(deprecated_rule.Rule):
    '''
    This rule has been split into:

    * `generic_map_008 <generic_map_rules.html#generic-map-008>`_
    * `port_map_008 <port_map_rules.html#port-map-008>`_
    '''
    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'instantiation', '024')
        self.message.append('Rule ' + self.unique_id + ' has been split into rules:')
        self.message.append('  generic_map_008')
        self.message.append('  rule port_map_005')




from vsg import parser

from vsg.vhdlFile.extract import tokens


def get_line_succeeding_line(iLine, lAllTokens, iNumLines, oTokenMap):
    lCarriageReturns = oTokenMap.get_token_indexes(parser.carriage_return)

    iStart = lCarriageReturns[iLine - 1] + 1
    try:
        iEnd = lCarriageReturns[iLine + iNumLines - 1]

        lTemp = lAllTokens[iStart:iEnd]

        return tokens.New(iStart, iLine + 1, lTemp)
    except IndexError:
        return None




from vsg.rules import insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment_and_not_on_same_line_as_token

from vsg.token import function_specification as token


class rule_009(insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment_and_not_on_same_line_as_token):
    '''
    This rule checks for a function parameter on the same line as the function keyword when the parameters are on multiple lines.

    **Violation**

    .. code-block:: vhdl

       function func_1 (a : integer; b : integer;
         c : unsigned(3 downto 0);
         d : std_logic_vector(7 downto 0);
         e : std_logic) return integer is
       begin

       end;


    **Fix**

    .. code-block:: vhdl

       function func_1 (
         a : integer; b : integer;
         c : unsigned(3 downto 0);
         d : std_logic_vector(7 downto 0);
         e : std_logic) return integer is
       begin

       end;
    '''

    def __init__(self):
        insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment_and_not_on_same_line_as_token.__init__(self, 'function', '009', token.open_parenthesis, token.close_parenthesis)
        self.solution = 'Move function parameter to the next line.'




from vsg.token import enumeration_type_definition as token

from vsg.vhdlFile import utils


def detect(iToken, lObjects):
    '''
    enumeration_type_definition ::=
        ( enumeration_literal { , enumeration_literal } )
    '''
    if utils.is_next_token('(', iToken, lObjects):
        return classify(iToken, lObjects)

    return iToken


def classify(iToken, lObjects):
    iCurrent = utils.assign_next_token_required('(', token.open_parenthesis, iToken, lObjects)

    while not utils.is_next_token(')', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_if(',', token.comma, iCurrent, lObjects)
        iCurrent = utils.assign_next_token(token.enumeration_literal, iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(')', token.close_parenthesis, iCurrent, lObjects)
    return iCurrent




from vsg import token

from vsg.rules.whitespace_between_token_pairs import Rule

lTokens = []
lTokens.append([token.component_instantiation_statement.label_colon, token.instantiated_unit.component_keyword])
lTokens.append([token.component_instantiation_statement.label_colon, token.instantiated_unit.entity_keyword])
lTokens.append([token.component_instantiation_statement.label_colon, token.instantiated_unit.configuration_keyword])
lTokens.append([token.component_instantiation_statement.label_colon, token.instantiated_unit.component_name])


class rule_002(Rule):
    '''
    This rule checks for a single space after the colon.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       U_FIFO :FIFO

    **Fix**

    .. code-block:: vhdl

       U_FIFO : FIFO
    '''
    def __init__(self):
        Rule.__init__(self, 'instantiation', '002', lTokens)




from vsg.vhdlFile.classify import interface_declaration


def classify(iToken, lObjects):
    '''
    interface_element ::=
        interface_declaration
    '''

    return interface_declaration.detect(iToken, lObjects)




from vsg import token

from vsg.rules.whitespace_between_token_pairs import Rule

lTokens = []
lTokens.append([token.use_clause.keyword, token.use_clause.library_name])


class rule_006(Rule):
    '''
    This rule checks for excessive spaces after the **use** keyword.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       library ieee;
         use    ieee.std_logic_1164.all;
         use   ieee.std_logic_unsigned.all;

    **Fix**

    .. code-block:: vhdl

       library ieee;
         use ieee.std_logic_1164.all;
         use ieee.std_logic_unsigned.all;
    '''
    def __init__(self):
        Rule.__init__(self, 'library', '006', lTokens)




from vsg.rules import token_case_with_prefix_suffix

from vsg import token

lTokens = []
lTokens.append(token.alias_declaration.alias_designator)


class rule_502(token_case_with_prefix_suffix):
    '''
    This rule checks the alias designator has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       alias Alias_Designator is name;

    **Fix**

    .. code-block:: vhdl

       alias alias_designator is name;
    '''

    def __init__(self):
        token_case_with_prefix_suffix.__init__(self, 'alias_declaration', '502', lTokens)
        self.groups.append('case::name')




from vsg import parser


class file_keyword(parser.keyword):
    '''
    unique_id = file_type_definition : file_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class of_keyword(parser.keyword):
    '''
    unique_id = file_type_definition : of_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)




from vsg.rules import blank_line_above_line_starting_with_token

from vsg.token import case_statement as token


class rule_009(blank_line_above_line_starting_with_token):
    '''
    This rule checks for blank lines or comments above the **end** keyword.

    |configuring_blank_lines_link|

    **Violation**

    .. code-block:: vhdl

         when others =>
           null;
       end case;

    **Fix**

    .. code-block:: vhdl

         when others =>
           null;

       end case;
    '''

    def __init__(self):
        blank_line_above_line_starting_with_token.__init__(self, 'case', '009', [token.end_keyword])




from vsg import parser

from vsg.rules import insert_tokens_right_of_token_if_it_does_not_exist_before_token

from vsg.token import package_body as token

lInsertTokens = []
lInsertTokens.append(token.end_package_keyword('package'))
lInsertTokens.append(parser.whitespace(' '))
lInsertTokens.append(token.end_body_keyword('body'))


class rule_002(insert_tokens_right_of_token_if_it_does_not_exist_before_token):
    '''
    This rule checks for the optional **package body** keywords on the end package body declaration.

    |configuring_optional_items_link|

    **Violation**

    .. code-block:: vhdl

       end FIFO_PKG;

    **Fix**

    .. code-block:: vhdl

       end package body FIFO_PKG;
    '''

    def __init__(self):
        insert_tokens_right_of_token_if_it_does_not_exist_before_token.__init__(self, 'package_body', '002', lInsertTokens, token.end_keyword, token.semicolon)
        self.solution = '*package body* keywords'




from vsg.rules import split_line_at_token

from vsg import token

lTokens = []
lTokens.append(token.use_clause.keyword)


class rule_011(split_line_at_token):
    '''
    This rule checks the **use** keyword is on its own line.

    **Violation**

    .. code-block:: vhdl

       context c1 is library ieee; use ieee.std_logic_1164.all; end context c1;

    **Fix**

    .. code-block:: vhdl

       context c1 is library ieee;
           use ieee.std_logic_1164.all; end context c1;
    '''

    def __init__(self):
        split_line_at_token.__init__(self, 'library', '011', lTokens)
        self.solution = 'Move *use* to it\'s own line.'




from vsg.rules import token_case_in_range_bounded_by_tokens

from vsg import token

lTokens = []
lTokens.append(token.generic_map_aspect.generic_keyword)
lTokens.append(token.generic_map_aspect.map_keyword)

oStart = token.component_instantiation_statement.label_colon
oEnd = token.component_instantiation_statement.semicolon


class rule_001(token_case_in_range_bounded_by_tokens):
    '''
    This rule checks the **generic map** keywords have proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       GENERIC MAP (

    **Fix**

    .. code-block:: vhdl

       generic map (
    '''

    def __init__(self):
        token_case_in_range_bounded_by_tokens.__init__(self, 'generic_map', '001', lTokens, oStart, oEnd)
        self.groups.append('case::keyword')




from vsg import token

from vsg.rules.whitespace_after_token import Rule

lTokens = []
lTokens.append(token.simple_waveform_assignment.assignment)
lTokens.append(token.simple_force_assignment.assignment)
lTokens.append(token.simple_release_assignment.assignment)
lTokens.append(token.selected_waveform_assignment.assignment)
lTokens.append(token.selected_force_assignment.assignment)
lTokens.append(token.conditional_waveform_assignment.assignment)
lTokens.append(token.conditional_force_assignment.assignment)


class rule_002(Rule):
    '''
    This rule checks for a single space after the **<=** operator.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       wr_en <=     '1';
       rd_en <='0';

    **Fix**

    .. code-block:: vhdl

       wr_en <= '1';
       rd_en <= '0';
    '''
    def __init__(self):
        Rule.__init__(self, 'sequential', '002', lTokens)




from .rule_001 import rule_001
from .rule_002 import rule_002
from .rule_003 import rule_003
from .rule_004 import rule_004
from .rule_005 import rule_005
from .rule_006 import rule_006
from .rule_007 import rule_007
from .rule_008 import rule_008
from .rule_009 import rule_009
from .rule_010 import rule_010
from .rule_011 import rule_011
from .rule_012 import rule_012
from .rule_013 import rule_013
from .rule_014 import rule_014

from .rule_016 import rule_016
from .rule_017 import rule_017
from .rule_018 import rule_018
from .rule_019 import rule_019
from .rule_020 import rule_020
from .rule_021 import rule_021
from .rule_022 import rule_022
from .rule_023 import rule_023
from .rule_024 import rule_024
from .rule_025 import rule_025
from .rule_026 import rule_026
from .rule_027 import rule_027
from .rule_028 import rule_028
from .rule_029 import rule_029
from .rule_030 import rule_030
from .rule_031 import rule_031
from .rule_032 import rule_032
from .rule_033 import rule_033
from .rule_034 import rule_034
from .rule_035 import rule_035

from .rule_600 import rule_600
from .rule_601 import rule_601




from vsg import parser

from vsg.token import selected_waveforms as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import choices
from vsg.vhdlFile.classify import waveform


def classify(iToken, lObjects):
    '''
    selected_waveforms ::=
        { waveform when choices , }
        waveform when choices
    '''
    iCurrent = iToken

    iCurrent = utils.assign_tokens_until('when', parser.todo, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('when', token.when_keyword, iCurrent, lObjects)
    sEnd = utils.find_earliest_occurance([',', ';'], iCurrent, lObjects)
    iCurrent = utils.assign_tokens_until(sEnd, parser.todo, iCurrent, lObjects)

    while utils.is_next_token(',', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required(',', token.comma, iCurrent, lObjects)
        iCurrent = utils.assign_tokens_until('when', parser.todo, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_required('when', token.when_keyword, iCurrent, lObjects)
        sEnd = utils.find_earliest_occurance([',', ';'], iCurrent, lObjects)
        iCurrent = utils.assign_tokens_until(sEnd, parser.todo, iCurrent, lObjects)

    return iCurrent


def classify_until(lUntils, iToken, lObjects):
    '''
    selected_waveforms ::=
        { waveform when choices , }
        waveform when choices
    '''
    iCurrent = iToken
    lMyUntils = lUntils
    lMyUntils.append(',')

    iCurrent = waveform.classify_until(['when'], iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('when', token.when_keyword, iCurrent, lObjects)
    iCurrent = choices.classify_until(lMyUntils, iCurrent, lObjects)

    while utils.is_next_token(',', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required(',', token.comma, iCurrent, lObjects)
        iCurrent = waveform.classify_until(['when'], iCurrent, lObjects)
        iCurrent = utils.assign_next_token_required('when', token.when_keyword, iCurrent, lObjects)
        iCurrent = choices.classify_until(lMyUntils, iCurrent, lObjects)

    return iCurrent




from .rule_100 import rule_100
from .rule_101 import rule_101

from .rule_300 import rule_300

from .rule_500 import rule_500
from .rule_501 import rule_501
from .rule_502 import rule_502
from .rule_503 import rule_503




error_type = 'error'
warning_type = 'warning'


class error():

    def __init__(self, name):
        self.name = name
        self.type = error_type


class warning():

    def __init__(self, name):
        self.name = name
        self.type = warning_type


class create_list():

    def __init__(self, dConfiguration):
        self.iMaxNameLength = 0
        self.lSeverities = _add_built_in_severities(self)
        _update_severities_from_configuration(self, dConfiguration)

    def add_severity(self, oSeverity):
        self.lSeverities.append(oSeverity)

    def get_severity_named(self, sName):
        for oSeverity in self.lSeverities:
            if sName == oSeverity.name:
                return oSeverity
        return None

    def get_severities(self):
        return self.lSeverities


def _add_built_in_severities(self):
    '''
    Creates the built in severity types.

    Parameters: (None)

    Returns: (list of severity objects)
    '''
    lReturn = []
    lReturn.append(error('Error'))
    lReturn.append(warning('Warning'))
    self.iMaxNameLength = max(self.iMaxNameLength, len('Warning'))
    return lReturn


def _extract_severities_from_configuration(dConfiguration):
    '''
    Creates a list of severities from a configuration.

    Parameters:

        dConfiguration: (Configuration dictionary)

    Returns: (list of severity objects)
    '''
    lReturn = []
    if 'severity' in dConfiguration:
        for sKey in list(dConfiguration['severity']):
            if dConfiguration['severity'][sKey]['type'] == error_type:
                lReturn.append(error(sKey))
            elif dConfiguration['severity'][sKey]['type'] == warning_type:
                lReturn.append(warning(sKey))
    return lReturn


def _update_severities_from_configuration(self, dConfiguration):
    '''
    Adds user defined severities to the build in severities.

    Parameters:

        dConfiguration: (Configuration dictionary)

    Returns: (Nothing)
    '''
    lUserSeverities = _extract_severities_from_configuration(dConfiguration)
    for oSeverity in lUserSeverities:
        self.add_severity(oSeverity)
        self.iMaxNameLength = max(self.iMaxNameLength, len(oSeverity.name))





from vsg.rules import insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment_when_between_tokens as Rule

from vsg import token

lTokens = []
lTokens.append(token.interface_list.semicolon)

lTokenPairs = []
lTokenPairs.append([token.generic_clause.open_parenthesis, token.generic_clause.close_parenthesis])


class rule_016(Rule):
    '''
    This rule checks for multiple generics defined on a single line.

    **Violation**

    .. code-block:: vhdl

      generic (
        g_width : std_logic := '0';g_depth : std_logic := '1'
      );

    **Fix**

    .. code-block:: vhdl

      generic (
        g_width : std_logic := '0';
        g_depth : std_logic := '1'
      );
    '''

    def __init__(self):
        Rule.__init__(self, 'generic', '016', lTokens, lTokenPairs)
        self.solution = 'Move multiple generics to their own lines.'




from vsg.rules import split_line_at_token

from vsg import token

lTokens = []
lTokens.append(token.if_statement.end_keyword)


class rule_020(split_line_at_token):
    '''
    This rule checks the **end if** keyword is on its own line.

    **Violation**

    .. code-block:: vhdl

       if (a = '1') then c <= '1'; else c <= '0'; end if;

    **Fix**

    .. code-block:: vhdl

       if (a = '1') then c <= '1'; else c <= '0';
       end if;
    '''

    def __init__(self):
        split_line_at_token.__init__(self, 'if', '020', lTokens)
        self.solution = 'Move *end if* keywords to their own line.'




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.procedure_specification.close_parenthesis)


class rule_006(token_indent):
    '''
    This rule checks the indent of the closing parenthesis if it is on its own line.

    **Violation**

    .. code-block:: vhdl

       procedure average_samples (
         constant a : in integer;
         signal d : out std_logic
         ) is


    **Fix**

    .. code-block:: vhdl

       procedure average_samples (
         constant a : in integer;
         signal d : out std_logic
       ) is
    '''

    def __init__(self):
        token_indent.__init__(self, 'procedure', '006', lTokens)




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.case_generate_alternative.when_keyword)


class rule_016(token_indent):
    '''
    This rule checks the indent of the **when** keyword in generate case statements.

    **Violation**

    .. code-block:: vhdl

       GEN_LABEL : case condition generate
         when 0 =>
           when 1 =>
       when 2 =>

    **Fix**

    .. code-block:: vhdl

       GEN_LABEL : case condition generate
         when 0 =>
         when 1 =>
         when 2 =>
    '''

    def __init__(self):
        token_indent.__init__(self, 'generate', '016', lTokens)




import math

from vsg import block_rule
from vsg import parser
from vsg import violation


class rule_001(block_rule.Rule):
    '''
    This rule checks the block comment header is correct.

    |configuring_block_comments_link|

    **Violation**

    .. code-block:: vhdl

       ----------------------------------------
       --   Comment
       --   Comment
       ----------------------------------------

    **Fix**

    .. code-block:: vhdl

       --+-------------[ Header ]==============
       --   Comment
       --   Comment
       ----------------------------------------
    '''

    def __init__(self):
        block_rule.Rule.__init__(self, 'block_comment', '001')
        self.header_left = None
        self.header_left_repeat = '-'
        self.header_string = None
        self.header_right_repeat = None
        self.header_alignment = 'center'
        self.max_header_column = 120
        self.configuration.extend(['header_left', 'header_left_repeat', 'header_string', 'header_right_repeat', 'header_alignment', 'max_header_column'])

    def analyze(self, oFile):

        self._print_debug_message('Analyzing rule: ' + self.unique_id)
        lToi = self._get_tokens_of_interest(oFile)

        for oToi in lToi:
            lTokens = oToi.get_tokens()

            if not self.allow_indenting:
                iWhitespace = 0
            elif isinstance(lTokens[0], parser.comment):
                iWhitespace = self.indentSize * lTokens[0].get_indent()
            else:
                iWhitespace = self.indentSize * lTokens[1].get_indent()

            sHeader = '--'

            if self.header_left is not None:
                sHeader += self.header_left
                iHeader_left = len(self.header_left)
            else:
                iHeader_left = 0

            if self.header_string is None:
                sHeader += self.header_left_repeat * (self.max_header_column - iWhitespace - len(sHeader))
            elif self.header_alignment == 'center':
                iLeft = math.floor((self.max_header_column - iWhitespace - len(self.header_string)) / 2) - iHeader_left - 2
                sLeft = self.header_left_repeat * iLeft
                iRight = self.max_header_column - iWhitespace - 2 - iHeader_left - len(self.header_string) - iLeft
                sRight = self.header_right_repeat * iRight
                sHeader += sLeft + self.header_string + sRight
            elif self.header_alignment == 'left':
                sHeader += self.header_left_repeat
                sHeader += self.header_string
                iLength = self.max_header_column - iWhitespace - len(sHeader)
                sHeader += self.header_right_repeat * iLength
            elif self.header_alignment == 'right':
                iLength = self.max_header_column - iWhitespace - len(sHeader) - len(self.header_string) - 1
                sHeader += self.header_left_repeat * iLength
                sHeader += self.header_string
                sHeader += self.header_right_repeat

            for oToken in lTokens:
                if isinstance(oToken, parser.comment):
                    sComment = oToken.get_value()
                    try:
                        if block_rule.is_header(sComment):
                            self.set_token_indent(oToken)
                            if sComment != sHeader:
                                sSolution = 'Change block comment header to : ' + sHeader
                                oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
                                self.add_violation(oViolation)
                        break
                    except IndexError:
                        break


#         1         2         3         4         5         6         7         8
#-------------------------------<-    80 chars    ->-----------------------------
#------------------------------<-    80 chars    ->------------------------------




from vsg.token import attribute_declaration as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import type_mark


def detect(iToken, lObjects):
    '''
    attribute_declaration ::=
        attribute identifier : type_mark ;
    '''
    if utils.are_next_consecutive_tokens(['attribute', None, ':'], iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('attribute', token.attribute_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token(token.identifier, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required(':', token.colon, iCurrent, lObjects)

    iCurrent = type_mark.classify(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.token import report_statement as token

from vsg.vhdlFile.classify import expression

from vsg.vhdlFile import utils


def detect(iToken, lObjects):
    '''
    report_statement ::=
        [ label : ]
            report expression
                [ severity expression ] ;
    '''

    if utils.keyword_found('report', iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.tokenize_label(iToken, lObjects, token.label, token.label_colon)
    iCurrent = utils.assign_next_token_required('report', token.report_keyword, iCurrent, lObjects)

    iCurrent = expression.classify_until([';', 'severity'], iCurrent, lObjects)

    if utils.is_next_token('severity', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('severity', token.severity_keyword, iCurrent, lObjects)
        iCurrent = expression.classify_until([';'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg import violation

from vsg.rule_group import structure


class existence_of_tokens_which_should_not_occur(structure.Rule):
    '''
    Checks the case for words.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : list of parser object types
       object types to check the prefix

    lPrefixes : string list
       acceptable prefixes
    '''

    def __init__(self, name, identifier, lTokens):
        structure.Rule.__init__(self, name=name, identifier=identifier)
        self.lTokens = lTokens
        self.fixable = False
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_matching(self.lTokens)

    def _analyze(self, lToi):
        for oToi in lToi:
            oViolation = violation.New(oToi.get_line_number(), oToi, self.solution)
            self.add_violation(oViolation)




from vsg.token import use_clause as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import utils as classify_utils


def detect(iToken, lObjects):
    '''
    use_clause ::=
        use selected_name { , selected_name } ;
    '''
    if utils.is_next_token('use', iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('use', token.keyword, iToken, lObjects)

    while not utils.is_next_token(';', iCurrent, lObjects):
        iCurrent = classify_utils.classify_selected_name(iCurrent, lObjects, token)
    
        if utils.is_next_token(',', iCurrent, lObjects):
            iCurrent = utils.assign_next_token_required(',', token.comma, iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)
    return iCurrent




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.if_statement.if_keyword)


class rule_001(token_indent):
    '''
    This rule checks the indent of the **if** keyword.

    **Violation**

    .. code-block:: vhdl

        if (a = '1') then
        b <= '0'
      elsif (c = '1') then
        d <= '1';
      else
        e <= '0';
      end if;

    **Fix**

    .. code-block:: vhdl

      if (a = '1') then
        b <= '0'
      elsif (c = '1') then
        d <= '1';
      else
        e <= '0';
      end if;
    '''

    def __init__(self):
        token_indent.__init__(self, 'if', '001', lTokens)




from vsg.token import conditional_expressions as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import condition
from vsg.vhdlFile.classify import expression


def classify_until(lUntils, iToken, lObjects):
    '''
    conditional_expressions ::=
        expression when condition
        { else expression when condition }
        [ else expression ]
    '''

    lMyElseUntils = lUntils.copy()
    lMyElseUntils.append('else')
    lMyWhenUntils = lUntils.copy()
    lMyWhenUntils.append('when')

    iCurrent = expression.classify_until(['when'], iToken, lObjects)
    iCurrent = utils.assign_next_token_required('when', token.when_keyword, iCurrent, lObjects)
    iCurrent = condition.classify_until(lMyElseUntils, iCurrent, lObjects)

    while utils.is_next_token('else', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('else', token.else_keyword, iCurrent, lObjects)
        iCurrent = expression.classify_until(lMyWhenUntils, iCurrent, lObjects)
        if utils.is_next_token_in_list(lUntils, iCurrent, lObjects):
            break
        iCurrent = utils.assign_next_token_required('when', token.when_keyword, iCurrent, lObjects)
        iCurrent = condition.classify_until(lMyElseUntils, iCurrent, lObjects)

    return iCurrent




from vsg.rules import align_tokens_in_region_between_tokens_unless_between_tokens as Rule

from vsg import prerequisite

from vsg import token

lAlign = []
lAlign.append(token.constant_declaration.assignment_operator)
lAlign.append(token.signal_declaration.assignment_operator)
lAlign.append(token.variable_declaration.assignment_operator)

lUnless = []
lUnless.append([token.subprogram_body.is_keyword, token.subprogram_body.begin_keyword])
lUnless.append([token.protected_type_body.body_keyword, token.protected_type_body.end_keyword])


class rule_400(Rule):
    '''
    This rule checks the alignment of **:=** operator for signal, constant and variable declarations.

    |configuring_keyword_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

       signal clk : std_logic := '0';
       variable reset : std_logic := '1';
       shared variable enable : std_logic := '0';
       constant reset_value : integer := 32;

    **Fix**

    .. code-block:: vhdl

       signal clk : std_logic             := '0';
       variable reset : std_logic         := '1';
       shared variable enable : std_logic := '0';
       constant reset_value : integer     := 32;
    '''

    def __init__(self):
        Rule.__init__(self, 'declarative_part', '400', lAlign, None, None, lUnless)
        self.solution = 'Align :='
        self.prerequisites.append(prerequisite.New('procedure_401'))
        self.prerequisites.append(prerequisite.New('architecture_026'))
        self.subphase = 3
        self.configuration.remove('if_control_statements_ends_group')
        self.configuration.remove('case_control_statements_ends_group')
        self.configuration.remove('loop_control_statements_ends_group')

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_in_declarative_parts()




from vsg.vhdlFile import utils

from vsg.token import block_statement as token

from vsg.vhdlFile.classify import block_declarative_part
from vsg.vhdlFile.classify import block_header
from vsg.vhdlFile.classify import block_statement_part


def detect(iToken, lObjects):
    '''
    block_statement ::=
    block_label :
        block [ ( *guard*_condition ) ] [ is ]
            block_header
            block_declarative_part
        begin
            block_statement_part
        end block [ block_label ] ;
    '''

    if utils.are_next_consecutive_tokens([None, ':', 'block'], iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.tokenize_label(iToken, lObjects, token.block_label, token.label_colon)
    iCurrent = utils.assign_next_token_required('block', token.block_keyword, iCurrent, lObjects)

    if utils.is_next_token('(', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('(', token.guard_open_parenthesis, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_if_not(')', token.guard_condition, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_required(')', token.guard_close_parenthesis, iCurrent, lObjects)

    iCurrent = utils.assign_next_token_if('is', token.is_keyword, iCurrent, lObjects)

    iCurrent = block_header.detect(iCurrent, lObjects)

    iCurrent = block_declarative_part.detect(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('begin', token.begin_keyword, iCurrent, lObjects)

    iCurrent = block_statement_part.detect(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('end', token.end_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('block', token.end_block_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if_not(';', token.end_block_label, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg import token

from vsg.rules import insert_token_left_of_token_if_it_does_not_exist_between_tokens


oInsertToken = token.instantiated_unit.component_keyword('component')

oAnchorToken = token.instantiated_unit.component_name

oStartToken = token.component_instantiation_statement.label_colon

oEndToken = token.component_instantiation_statement.semicolon


class rule_033(insert_token_left_of_token_if_it_does_not_exist_between_tokens):
    '''
    This rule checks for the **component** keyword for a component instantiation.

    |configuring_optional_items_link|

    **Violation**

    .. code-block:: vhdl

       INSTANCE_NAME : ENTITY_NAME

    **Fix**

    .. code-block:: vhdl

       INSTANCE_NAME : component ENTITY_NAME
    '''
    def __init__(self):
        insert_token_left_of_token_if_it_does_not_exist_between_tokens.__init__(self, 'instantiation', '033', oInsertToken, oAnchorToken, oStartToken, oEndToken)
        self.solution = '*component* keyword'
        self.groups.append('structure::optional')




from vsg.token import interface_unknown_declaration as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import expression
from vsg.vhdlFile.classify import identifier_list
from vsg.vhdlFile.classify import mode
from vsg.vhdlFile.classify import subtype_indication


def detect(iToken, lObjects):
    '''
        This is a classification if the signal, constant, or variable keywords can not be found.
        This is not in the VHDL LRM.
        It is based off the interface_signal_declaration as it has the most keywords.

        interface_unknown_declaration ::=
            identifier_list : [ mode ] subtype_indication [ bus ] [ := *static*_expression ]
    '''

    if utils.is_next_token_one_of(['type', 'file', 'function', 'procedure', 'impure', 'pure', 'package'], iToken, lObjects):
        return iToken
    else:
        return classify(iToken, lObjects)


def classify(iToken, lObjects):

    iCurrent = identifier_list.classify_until([':'], iToken, lObjects, token.identifier)

    iCurrent = utils.assign_next_token_required(':', token.colon, iCurrent, lObjects)

    iCurrent = mode.classify(iCurrent, lObjects)

    iCurrent = subtype_indication.classify(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_if('bus', token.bus_keyword, iCurrent, lObjects)

    if utils.is_next_token(':=', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required(':=', token.assignment, iCurrent, lObjects)

        iCurrent = expression.classify_until([';'], iCurrent, lObjects)

    return iCurrent




from vsg.vhdlFile import utils

from vsg.token import for_generate_statement as token

from vsg.vhdlFile.classify import parameter_specification
from vsg.vhdlFile.classify import generate_statement_body


def detect(iToken, lObjects):
    '''
    for_generate_statement ::=
        *generate*_label :
            for *generate*_parameter_specification generate
                generate_statement_body
            end generate [ *generate*_label ] ;
    '''

    if utils.are_next_consecutive_tokens([None, ':', 'for'], iToken, lObjects):
        return classify(iToken, lObjects)
    if utils.are_next_consecutive_tokens(['for'], iToken, lObjects):
        iIndex = utils.find_next_token(iToken, lObjects)
        oToken = token.for_keyword(lObjects[iToken].get_value())
        utils.print_error_message('generate_label', oToken, iIndex, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.tokenize_label(iToken, lObjects, token.generate_label, token.label_colon)

    iCurrent = utils.assign_next_token_required('for', token.for_keyword, iCurrent, lObjects)

    iCurrent = parameter_specification.classify_until(['generate'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('generate', token.generate_keyword, iCurrent, lObjects)

    iCurrent = generate_statement_body.classify(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('end', token.end_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('generate', token.end_generate_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if_not(';', token.end_generate_label, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent





from vsg import violation

from vsg.rule_group import naming


class is_token_value_one_of(naming.Rule):
    '''
    Checks if a token value is in a list of provided values.
    '''

    def __init__(self, name, identifier, token):
        naming.Rule.__init__(self, name, identifier)
        self.names = []
        self.fixable = False
        self.disable = True
        self.configuration.append('names')
        self.token = token
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_matching([self.token])

    def _analyze(self, lToi):
        lower_names = []
        for sName in self.names:
            lower_names.append(sName.lower())

        for oToi in lToi:
            lTokens = oToi.get_tokens()
            if not lTokens[0].get_value().lower() in lower_names:
                self.add_violation(violation.New(oToi.get_line_number(), oToi, self.solution))




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.attribute_declaration.attribute_keyword)


class rule_300(token_indent):
    '''
    This rule checks the indent of the **attribute** keyword.

    **Violation**

    .. code-block:: vhdl

       signal sig1 : std_logic;
          attribute max_delay : time;

    **Fix**

    .. code-block:: vhdl

       signal sig1 : std_logic;
       attribute max_delay : time;
    '''

    def __init__(self):
        token_indent.__init__(self, 'attribute_declaration', '300', lTokens)




from vsg import parser
from vsg import severity
from vsg import violation

from vsg.rule_group import structure


class rule_012(structure.Rule):
    '''
    This rule checks for user defined keywords in comments.

    .. NOTE:: This rule is disabled by default.

    |configuring_comment_keywords_link|

    **Violation**

    .. code-block:: vhdl

       -- TODO:  Refactor the section below
       -- FIXME: Update

    **Fix**

    This is a reporting only rule.
    '''

    def __init__(self):
        structure.Rule.__init__(self, name='comment', identifier='012')
        self.solution = 'Move inline comment to previous line.'
        self.disable = True
        self.fixable = False
        self.severity = severity.warning('Warning')
        self.keywords = ['TODO', 'FIXME']
        self.configuration.append('keywords')
        self.configuration_documentation_link = 'configuring_comment_keywords_link'

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_matching([parser.comment])

    def _analyze(self, lToi):
        for oToi in lToi:
            if self.comment_includes_keyword(oToi):
                self.create_violation(oToi)

    def comment_includes_keyword(self, oToi):
        sToken = oToi.get_tokens()[0].get_value()
        for sKeyword in self.keywords:
            if sKeyword in sToken:
                oToi.set_meta_data('keyword', sKeyword)
                return True
        return False

    def create_violation(self, oToi):
        sSolution = 'Comment keyword ' + oToi.get_meta_data('keyword') + ' detected.'
        oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
        self.add_violation(oViolation)




from vsg.token import instantiated_unit as token

from vsg.vhdlFile import utils


def detect(iCurrent, lObjects):
    '''
    instantiated_unit ::=
        [ component ] component_name
      | entity entity_name [ ( *architecture*_identifier ) ]
      | configuration configuration_name
    '''
    iToken = iCurrent
    if utils.is_next_token_one_of(['component', 'entity', 'configuration'], iToken, lObjects):
        return True
    if utils.find_in_next_n_tokens(';', 2, iToken, lObjects):
        return True
    # Check if this is a signal assignment
    if utils.find_in_range('<=', iToken, ';', lObjects):
        return False
    if utils.find_in_range('generate', iToken, ';', lObjects):
        return False
    return True

def classify(iToken, lObjects):

    iCurrent = iToken

    if utils.is_next_token('component', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('component', token.component_keyword, iCurrent, lObjects)
        iCurrent = utils.assign_next_token(token.component_name, iCurrent, lObjects)

    elif utils.is_next_token('configuration', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('configuration', token.configuration_keyword, iCurrent, lObjects)
        iCurrent = utils.assign_next_token(token.configuration_name, iCurrent, lObjects)

    elif utils.is_next_token('entity', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('entity', token.entity_keyword, iCurrent, lObjects)

        iCurrent = classify_entity_name(iCurrent, lObjects)

        if utils.is_next_token('(', iCurrent, lObjects):
            iCurrent = utils.assign_next_token_required('(', token.open_parenthesis, iCurrent, lObjects)
            iCurrent = utils.assign_next_token_if_not(')', token.architecture_identifier, iCurrent, lObjects)
            iCurrent = utils.assign_next_token_required(')', token.close_parenthesis, iCurrent, lObjects)
    else:
        iCurrent = utils.assign_next_token(token.component_name, iCurrent, lObjects)

    return iCurrent


def classify_entity_name(iToken, lObjects):
    iCurrent = utils.find_next_token(iToken, lObjects)
    sTokenValue = lObjects[iCurrent].get_value()
    if '.' in sTokenValue:
        lTokenValue = sTokenValue.split('.')
        lObjects[iCurrent] = token.library_name(lTokenValue[0])
        lObjects.insert(iCurrent + 1, token.dot('.'))
        lObjects.insert(iCurrent + 2, token.entity_name(lTokenValue[1]))
        iCurrent = iCurrent + 2
    else:
        iCurrent = utils.assign_next_token(token.entity_name, iCurrent, lObjects)

    return iCurrent





from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import conditional_waveform_assignment
from vsg.vhdlFile.classify import conditional_force_assignment


def detect(iToken, lObjects):
    '''
    conditional_signal_assignment ::=
        conditional_waveform_assignment
      | conditional_force_assignment
    '''

    if utils.is_next_token('when', iToken, lObjects):
        return False
    if utils.find_in_next_n_tokens('if', 3, iToken, lObjects):
        return False
    if utils.find_in_range('<=', iToken, ';', lObjects):
        if utils.find_in_range('when', iToken, ';', lObjects):
            return True
    return False


def classify(iToken, lObjects):

    iCurrent = conditional_force_assignment.detect(iToken, lObjects)
    if iCurrent != iToken:
        return iCurrent

    iCurrent = conditional_waveform_assignment.detect(iToken, lObjects)

    return iCurrent




from vsg.rules.whitespace_before_token_if_on_same_line_as_token import Rule

from vsg.token import alias_declaration as token


class rule_101(Rule):
    '''
    This rule checks for a single space before the **is** keyword if the : is present.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       alias alias_designator : subtype_indication     is name;
       alias alias_designator   is name;

    **Fix**

    .. code-block:: vhdl

       alias alias_designator : subtype_indication is name;
       alias alias_designator   is name;
    '''
    def __init__(self):
        Rule.__init__(self, 'alias_declaration', '101')
        self.oFirstToken = token.is_keyword
        self.oExceptToken = token.colon




from vsg.vhdlFile.classify import expression


def classify(iToken, lObjects):
    '''
    generate_specification ::=
        static_discrete_range
      | static_expression
      | alternative_label
    '''

    return expression.classify_until([')'], iToken, lObjects)




from vsg import deprecated_rule


class rule_001(deprecated_rule.Rule):
    '''
    This rule has been replaced with rules in the selected_assignment group.
    '''

    def __init__(self):

        deprecated_rule.Rule.__init__(self, 'with', '001')
        self.message.append('Rule ' + self.unique_id + ' has been replaced by selected_assignment rules.')



from vsg.rules import previous_line

from vsg import token

lTokens = []
lTokens.append(token.component_instantiation_statement.instantiation_label)


class rule_004(previous_line):
    '''
    This rule checks for blank lines or comments above the instantiation.

    |configuring_previous_line_rules_link|

    The default style is :code:`no_code`.

    **Violation**

    .. code-block:: vhdl

       WR_EN <= '1';
       U_FIFO : FIFO

       -- Instantiate another FIFO
       U_FIFO2 : FIFO

    **Fix**

    .. code-block:: vhdl

       WR_EN <= '1';

       U_FIFO : FIFO

       -- Instantiate another FIFO
       U_FIFO2 : FIFO
    '''

    def __init__(self):
        previous_line.__init__(self, 'instantiation', '004', lTokens)
        self.style = 'no_code'





from vsg import parser
from vsg import violation

from vsg.rules import utils as rules_utils
from vsg.rule_group import structure
from vsg.vhdlFile import utils


class move_token_right_to_next_non_whitespace_token(structure.Rule):
    '''
    Moves one token to the right until it encounters a non whitespace token.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    tokens_to_move : token type
       The token which will be moved next to the anchor token.
    '''

    def __init__(self, name, identifier, tokens_to_move):
        structure.Rule.__init__(self, name, identifier)
        self.tokens_to_move = tokens_to_move
        self.bInsertWhitespace = True
        self.bRemoveTrailingWhitespace = True
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        lToi = []
        for oToken in self.tokens_to_move:
            aToi = oFile.get_tokens_from_beginning_of_line_containing_token_to_the_next_non_whitespace_token_to_the_right(oToken)
            lToi = utils.combine_two_token_class_lists(lToi, aToi)
        lReturn = filter_toi(self, lToi)
        return lReturn

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            iTokenIndex = oToi.get_meta_data('iTokenIndex')

            sSolution = 'Move ' + lTokens[iTokenIndex].get_value() + ' to same line as ' + lTokens[-1].get_value()
            oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
            oViolation.set_remap()
            oViolation.set_action(oToi.get_meta_data('iTokenIndex'))
            oViolation.fix_blank_lines = True
            self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        iTokenIndex = oViolation.get_action()
        rules_utils.insert_token(lTokens, -1, lTokens.pop(iTokenIndex))

        if self.bInsertWhitespace:
            rules_utils.insert_whitespace(lTokens, -1)

        lNewTokens = utils.remove_consecutive_whitespace_tokens(lTokens)
        lNewTokens = utils.remove_all_trailing_whitespace(lNewTokens)

        lNewTokens = utils.fix_blank_lines(lNewTokens)
        oViolation.set_tokens(lNewTokens)


def tokens_are_next_to_each_other(oToi):
    lTokens = oToi.get_tokens()
    iTokenIndex = oToi.get_meta_data('iTokenIndex')

    if len(lTokens) - 2 == iTokenIndex:
        return True
    return False


def filter_toi(self, lToi):
    lReturn = []
    for oToi in lToi:
        if tokens_are_next_to_each_other(oToi):
            continue
        if skip_based_on_whitespace(self.bInsertWhitespace, oToi):
            continue
        lReturn.append(oToi)
    return lReturn


def skip_based_on_whitespace(bInsertWhitespace, oToi):
    if bInsertWhitespace and does_a_whitespace_token_separate_tokens(oToi):
        return True
    return False


def does_a_whitespace_token_separate_tokens(oToi):
    lTokens = oToi.get_tokens()
    iTokenIndex = oToi.get_meta_data('iTokenIndex')

    if len(lTokens) - iTokenIndex == 3 and isinstance(lTokens[-2], parser.whitespace):
        return True
    return False




from vsg.token import interface_function_specification as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import formal_parameter_list
from vsg.vhdlFile.classify import type_mark


def detect(iToken, lObjects):
    '''
    interface_function_specification ::=
        [ pure | impure ] function designator
            [ [ parameter ] ( formal_parameter_list ) ] return type_mark
    '''

    if utils.is_next_token('pure', iToken, lObjects):
        return classify(iToken, lObjects)
    elif utils.is_next_token('impure', iToken, lObjects):
        return classify(iToken, lObjects)
    elif utils.is_next_token('function', iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_if('pure', token.pure_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token_if('impure', token.impure_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('function', token.function_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token(token.designator, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if('parameter', token.parameter_keyword, iCurrent, lObjects)

    if utils.is_next_token('(', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('(', token.open_parenthesis, iCurrent, lObjects)

        iCurrent = formal_parameter_list.classify(iCurrent, lObjects)

        iCurrent = utils.assign_next_token_required(')', token.close_parenthesis, iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('return', token.return_keyword, iToken, lObjects)
    iCurrent = type_mark.classify(iToken, lObjects)

    return iCurrent




from vsg.token import assertion as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import condition
from vsg.vhdlFile.classify import expression


def classify(iToken, lObjects):
    '''
    assertion ::=
        assert condition
            [ report expression ]
            [ severity expression ]

    The key to detecting this is looking for the keyword **assert** before a semicolon.
    '''

    iCurrent = utils.assign_next_token_required('assert', token.keyword, iToken, lObjects)

    iCurrent = condition.classify_until(['report', 'severity', ';'], iCurrent, lObjects)

    if utils.is_next_token('report', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('report', token.report_keyword, iCurrent, lObjects)
        iCurrent = expression.classify_until(['severity', ';'], iCurrent, lObjects)

    if utils.is_next_token('severity', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('severity', token.severity_keyword, iCurrent, lObjects)
        iCurrent = expression.classify_until([';'], iCurrent, lObjects)

    return iCurrent




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.signal_declaration.signal_keyword)


class rule_002(token_case):
    '''
    This rule checks the **signal** keyword has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       SIGNAL wr_en : std_logic;

    **Fix**

    .. code-block:: vhdl

       signal wr_en : std_logic;
    '''

    def __init__(self):
        token_case.__init__(self, 'signal', '002', lTokens)
        self.groups.append('case::keyword')




from vsg import token

from vsg.rules import remove_carriage_return_after_token

lTokens = []
lTokens.append(token.if_statement.if_keyword)
lTokens.append(token.if_statement.elsif_keyword)


class rule_035(remove_carriage_return_after_token):
    '''
    This rule checks the expression after the **if** or **elsif** keyword starts on the same line.

    **Violation**

    .. code-block:: vhdl

       if
         a = '1' then

       elsif
         b = '1' then

    **Fix**

    .. code-block:: vhdl

       if a = '1' then

       elsif b = '1' then
    '''

    def __init__(self):
        remove_carriage_return_after_token.__init__(self, 'if', '035', lTokens, bInsertSpace=True)
        self.solution = 'Merge lines with **elsif** keyword and expression below.'




from vsg.rules import previous_line

from vsg.token import use_clause as token

lTokens = []
lTokens.append(token.keyword)


class rule_007(previous_line):
    '''
    This rule checks for blank lines or comments above the **use** declaration.

    |configuring_previous_line_rules_link|

    The default style is :code:`no_blank_line`.

    **Violation**

    .. code-block:: vhdl

       library ieee;

         use ieee.std_logic_1164.all;

         use ieee.std_logic_unsigned.all;

    **Fix**

    .. code-block:: vhdl

       library ieee;
         use ieee.std_logic_1164.all;
         use ieee.std_logic_unsigned.all;
    '''
    def __init__(self):
        previous_line.__init__(self, 'library', '007', lTokens)
        self.style = 'no_blank_line'




from vsg.rules import is_token_value_one_of

from vsg.token import architecture_body as token


class rule_025(is_token_value_one_of):
    '''
    This rule checks for valid names for the architecture.
    Typical architecture names are:  RTL, EMPTY, and BEHAVE.
    This rule allows the user to restrict what can be used for an architecture name.

    .. NOTE:: This rule is disabled by default.
       You can enable and configure the names using the following configuration.

       .. code-block:: yaml

          ---

          rule :
            architecture_025 :
              disable : False
              names :
                - rtl
                - empty
                - behave

    **Violation**

    .. code-block:: vhdl

       architecture some_invalid_arch_name of entity1 is

    **Fix**

    The user is required to decide which is the correct architecture name.
    '''

    def __init__(self):
        is_token_value_one_of.__init__(self, 'architecture', '025', token.identifier)

    def _get_solution(self, iLineNumber):
        return 'Architecture identifier must be from this list: ' + ', '.join(self.names)




from vsg.token import package_body as token

from vsg.vhdlFile.extract.get_tokens_bounded_by import get_tokens_bounded_by

oStart = token.is_keyword
oEnd = token.end_keyword


def extract(lAllTokens, oTokenMap):

    return get_tokens_bounded_by(oStart, oEnd, lAllTokens, oTokenMap)




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.subprogram_kind.procedure_keyword)


class rule_009(token_case):
    '''
    This rule checks the **procedure** keyword in the **end procedure** has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       end PROCEDURE;

       end Procedure proc;

    **Fix**

    .. code-block:: vhdl

       end procedure;

       end procedure proc;
    '''

    def __init__(self):
        token_case.__init__(self, 'procedure', '009', lTokens)
        self.groups.append('case::keyword')




from vsg import proposed_rule


class rule_009(proposed_rule.Rule):
    '''
    This rule checks for multiple selected names in a single reference.

    .. NOTE:: This rule has not been implemented yet.

    **Violation**

    .. code-block:: vhdl

       context c1, c2, c3; -- Comment 1

       context c1,
               c2,
               c3;

    .. code-block:: vhdl

       context c1;
       context c2;
       context c3;

       context c1;
       context c2;
       context c3;
    '''

    def __init__(self):
        proposed_rule.Rule.__init__(self, 'context_ref', '009')




from vsg.vhdlFile.extract import tokens


def get_sequence_of_tokens_not_matching(lTokens, lAllTokens, oTokenMap):

    lReturn = []
    lIndexes = oTokenMap.get_token_indexes(lTokens[0])

    for iIndex in lIndexes:
        for i in range(1, len(lTokens)):
            if not oTokenMap.is_token_at_index(lTokens[i], iIndex + i):
                iLine = oTokenMap.get_line_number_of_index(iIndex)
                lReturn.append(tokens.New(iIndex, iLine, lAllTokens[iIndex:iIndex + 1]))
                break
    return lReturn




from vsg.rules import token_prefix

from vsg import token

lTokens = []
lTokens.append(token.signal_declaration.identifier)


class rule_008(token_prefix):
    '''
    This rule checks for valid prefixes on signal identifiers.
    Default signal prefix is *s\_*.

    |configuring_prefix_and_suffix_rules_link|

    **Violation**

    .. code-block:: vhdl

       signal wr_en : std_logic;
       signal rd_en : std_logic;

    **Fix**

    .. code-block:: vhdl

       signal s_wr_en : std_logic;
       signal s_rd_en : std_logic;
    '''

    def __init__(self):
        token_prefix.__init__(self, 'signal', '008', lTokens)
        self.prefixes = ['s_']
        self.solution = 'Signal identifiers'




from vsg import token

from vsg.rules.whitespace_between_token_pairs import Rule

lTokens = []
lTokens.append([token.procedure_specification.procedure_keyword, token.procedure_specification.designator])
lTokens.append([token.procedure_specification.designator, token.subprogram_body.is_keyword])
lTokens.append([token.procedure_specification.designator, token.procedure_specification.open_parenthesis])
lTokens.append([token.procedure_specification.close_parenthesis, token.subprogram_body.is_keyword])

class rule_100(Rule):
    '''
    This rule checks for a single space between the following procedure elements:  **procedure** keyword, procedure designator, open parenthesis, close parenthesis, and **is** keywords.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       procedure    average_samples    (
           constant a : in integer;
           signal d : out std_logic
         )    is
       procedure    average_samples      is

    **Fix**

    .. code-block:: vhdl

       procedure average_samples (
           constant a : in integer;
           signal d : out std_logic
         ) is
       procedure average_samples is
    '''
    def __init__(self):
        Rule.__init__(self, 'procedure', '100', lTokens)




from vsg.token import interface_constant_declaration as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import identifier_list
from vsg.vhdlFile.classify import subtype_indication
from vsg.vhdlFile.classify import expression
from vsg.vhdlFile.classify import mode


def detect(iToken, lObjects):
    '''
    interface_constant_declaration ::=
    [ constant ] identifier_list : [ in ] subtype_indication [ := static_expression ]
    '''

    if utils.is_next_token('constant', iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('constant', token.constant_keyword, iToken, lObjects)

    iCurrent = identifier_list.classify_until([':'], iCurrent, lObjects, token.identifier)

    iCurrent = utils.assign_next_token_required(':', token.colon, iCurrent, lObjects)

    iCurrent = mode.classify(iCurrent, lObjects)

    iCurrent = subtype_indication.classify(iCurrent, lObjects)

    if utils.is_next_token(':=', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required(':=', token.assignment, iCurrent, lObjects)
        iCurrent = expression.classify_until([';'], iCurrent, lObjects)

    return iCurrent




from vsg import parser

from vsg.rules import remove_spaces_before_token_rule


class rule_004(remove_spaces_before_token_rule):
    '''
    This rule checks for spaces before commas.

    **Violation**

    .. code-block:: vhdl

       wr_en => wr_en    ,
       rd_en => rd_en,

    **Fix**

    .. code-block:: vhdl

       wr_en => wr_en,
       rd_en => rd_en,
    '''

    def __init__(self):
        remove_spaces_before_token_rule.__init__(self, 'whitespace', '004', parser.comma)
        self.solution = 'Remove spaces before commas.'
        self.configuration_documentation_link = None




from vsg import parser

from vsg.token import choice as token

from vsg.vhdlFile import utils


def classify_until(lUntils, iToken, lObjects):
    '''
    choice ::=
        simple_expression
      | discrete_range
      | *element*_simple_name
      | others
    '''

    for iIndex in range(iToken, len(lObjects)):
        if not utils.is_item(lObjects, iIndex):
            continue
        if utils.is_next_token_in_list(lUntils, iIndex, lObjects):
            return iIndex
        else:
            if utils.is_next_token('others', iIndex, lObjects):
                utils.assign_next_token_required('others', token.others_keyword, iIndex, lObjects)
            else:
                utils.assign_next_token(parser.todo, iIndex, lObjects)

    return iToken




from vsg.rules import multiline_constraint_structure as Rule

from vsg import token

lTokenPairs = []
lTokenPairs.append([token.signal_declaration.signal_keyword, token.signal_declaration.semicolon])


class rule_017(Rule):
    '''
    This rule checks the structure of signal constraints.

    |configuring_multiline_constraint_rules_link|

    .. NOTE:: The indenting of multiline signal constraints is handled by the rule `signal_400 <signal_rules.html#signal-400>`_.

    **Violation**

    .. code-block:: vhdl

       signal sig_a : my_record(element1(7 downto 0), element2(3 downto 0));

    **Fix**

    .. code-block:: vhdl

       signal sig_a : my_record(
           element1(7 downto 0),
           element2(3 downto 0)
         );
    '''

    def __init__(self):
        Rule.__init__(self, 'signal', '017')
        self.lTokenPairs = lTokenPairs




from vsg.rules import align_tokens_in_region_between_tokens_unless_between_tokens

from vsg import token

lAlign = []
lAlign.append(token.simple_waveform_assignment.assignment)
lAlign.append(token.simple_force_assignment.assignment)
lAlign.append(token.simple_release_assignment.assignment)
lAlign.append(token.simple_variable_assignment.assignment)
lAlign.append(token.conditional_variable_assignment.assignment)

oStart = token.process_statement.begin_keyword
oEnd = token.process_statement.end_keyword

lUnless = []


class rule_400(align_tokens_in_region_between_tokens_unless_between_tokens):
    '''
    This rule checks the alignment of the **<=** and **:=** operators over consecutive sequential assignments in the process_statement_part.

    Following extra configurations are supported:

    * :code:`if_control_statements_ends_group`,
    * :code:`case_control_statements_ends_group`.
    * :code:`case_keyword_statements_ends_group`.
    * :code:`loop_control_statements_ends_group`,

    |configuring_keyword_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

       wr_en <= '1';
       rd_en   <= '0';
       v_variable := 10;

    **Fix**

    .. code-block:: vhdl

       wr_en      <= '1';
       rd_en      <= '0';
       v_variable := 10;
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens_unless_between_tokens.__init__(self, 'process', '400', lAlign, oStart, oEnd, lUnless)
        self.solution = 'Align identifer.'
        self.if_control_statements_ends_group = True
        self.case_control_statements_ends_group = True
        self.case_keyword_statements_ends_group = True
        self.loop_control_statements_ends_group = True




from vsg import token

from vsg.rules import previous_line

lTokens = []
lTokens.append(token.incomplete_type_declaration.type_keyword)
lTokens.append(token.full_type_declaration.type_keyword)


class rule_010(previous_line):
    '''
    This rule checks for blank lines or comments above the **type** declaration.

    |configuring_previous_line_rules_link|

    **Violation**

    .. code-block:: vhdl

       signal wr_en : std_logic;
       type state_machine is (idle, write, read, done);

    **Fix**

    .. code-block:: vhdl

       signal wr_en : std_logic;

       type state_machine is (idle, write, read, done);
    '''

    def __init__(self):
        previous_line.__init__(self, 'type', '010', lTokens)




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.incomplete_type_declaration.type_keyword)
lTokens.append(token.full_type_declaration.type_keyword)


class rule_001(token_indent):
    '''
    This rule checks the indent of the **type** declaration.

    **Violation**

    .. code-block:: vhdl

       architecture rtl of fifo is

           type state_machine is (idle, write, read, done);

       begin

    **Fix**

    .. code-block:: vhdl

       architecture rtl of fifo is

         type state_machine is (idle, write, read, done);

       begin
    '''

    def __init__(self):
        token_indent.__init__(self, 'type', '001', lTokens)




from vsg import deprecated_rule


class rule_012(deprecated_rule.Rule):
    '''
    This rule was a duplicate of whitespace_200 and has been removed.
    '''

    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'whitespace', '012')
        self.message.append('Rule ' + self.unique_id + ' has been replaced with whitespace_200.')




from vsg import parser
from vsg import violation

from vsg.rule_group import whitespace


class rule_001(whitespace.Rule):
    '''
    This rule check for trailing spaces.

    **Violation**

    Where periods indicate spaces:

    .. code-block:: vhdl

        library ieee;....

    **Fix**

    .. code-block:: vhdl

        library ieee;
    '''

    def __init__(self):

        whitespace.Rule.__init__(self, 'whitespace', '001')
        self.phase = 1
        self.subphase = 0
        self.solution = 'Remove trailing whitespace'
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_n_tokens_before_token(2, [parser.carriage_return])

    def _analyze(self, lToi):
        for oToi in lToi:
            if whitespace_exists(oToi):
                create_violation(oToi, self)

    def _fix_violation(self, oViolation):
        dAction = oViolation.get_action()
        if need_to_remove_whitespace(dAction):
            remove_whitespace(oViolation)
        else:
            remove_whitespace_and_insert_blank_line(oViolation)


def create_violation(oToi, self):
    lTokens = oToi.get_tokens()
    oViolation = violation.New(oToi.get_line_number(), oToi, self.solution)
    dAction = define_action(lTokens)
    oViolation.set_action(dAction)
    self.add_violation(oViolation)


def whitespace_exists(oToi):
    lTokens = oToi.get_tokens()
    if isinstance(lTokens[1], parser.whitespace):
        return True
    return False


def define_action(lTokens):
    dAction = {}
    if isinstance(lTokens[0], parser.carriage_return):
        dAction['action'] = 'insert_blank_line'
    else:
        dAction['action'] = 'remove'
    return dAction


def need_to_remove_whitespace(dAction):
    if dAction['action'] == 'remove':
        return True
    return False


def remove_whitespace(oViolation):
    lTokens = oViolation.get_tokens()
    myTokens = []
    myTokens.append(lTokens[0])
    myTokens.append(lTokens[-1])
    oViolation.set_tokens(myTokens)


def remove_whitespace_and_insert_blank_line(oViolation):
    lTokens = oViolation.get_tokens()
    myTokens = []
    myTokens.append(lTokens[0])
    myTokens.append(parser.blank_line())
    myTokens.append(lTokens[-1])
    oViolation.set_tokens(myTokens)




from vsg import parser


class assignment(parser.assignment):
    '''
    unique_id = association_element : assignment
    '''

    def __init__(self, sString):
        parser.item.__init__(self, sString)


class formal_part(parser.item):
    '''
    unique_id = association_element : formal_part
    '''

    def __init__(self, sString):
        parser.item.__init__(self, sString)


class actual_part(parser.item):
    '''
    unique_id = association_element : actual_part
    '''

    def __init__(self, sString):
        parser.item.__init__(self, sString)




from vsg import parser
from vsg import violation

from vsg.vhdlFile import utils
from vsg.rule_group import whitespace

lTokens = []
lTokens.append(parser.open_parenthesis)


class rule_005(whitespace.Rule):
    '''
    This rule checks for spaces after an open parenthesis.

    .. NOTE::
       Spaces before numbers are allowed.

    **Violation**

    .. code-block:: vhdl

       signal data        : std_logic_vector(31 downto 0);
       signal byte_enable : std_logic_vector( 3 downto 0);
       signal width       : std_logic_vector(  g_width - 1 downto 0);

    **Fix**

    .. code-block:: vhdl

       signal data        : std_logic_vector(31 downto 0);
       signal byte_enable : std_logic_vector( 3 downto 0);
       signal width       : std_logic_vector(g_width - 1 downto 0);
    '''

    def __init__(self):
        whitespace.Rule.__init__(self, 'whitespace', '005')
        self.solution = 'Remove spaces after open (.'
        self.iSpaces = 0
        self.lTokens = [parser.open_parenthesis]
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_n_tokens_before_and_after_tokens(2, self.lTokens)

    def _analyze(self, lToi):

        for oToi in lToi:
            iLine, lTokens = utils.get_toi_parameters(oToi)

            oRight = lTokens[-2]
            if isinstance(oRight, parser.whitespace):
                if not utils.token_is_whitespace_or_comment(lTokens[-1]):
                    if not lTokens[-1].get_value().isnumeric():
                        oViolation = violation.New(iLine, oToi, self.solution)
                        self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        oToken = lTokens.pop()
        lTokens.pop()
        lTokens.append(oToken)
        oViolation.set_tokens(lTokens)





from vsg import parser
from vsg import violation

from vsg.vhdlFile import utils
from vsg.rule_group import whitespace
from vsg.rules import utils as rules_utils


class spaces_before_and_after_tokens_when_bounded_by_tokens(whitespace.Rule):
    '''
    Checks for a single space between two tokens.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : list of token type pairs
       The tokens to check for a single space between
    '''

    def __init__(self, name, identifier, lTokens, lBetween):
        whitespace.Rule.__init__(self, name=name, identifier=identifier)
        self.lTokens = lTokens
        self.spaces_before = 1
        self.configuration.append('spaces_before')
        self.spaces_after = 4
        self.configuration.append('spaces_after')
        self.lBetween = lBetween
        self.nTokens = 2

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_n_tokens_before_and_after_tokens_bounded_by_tokens(self.nTokens, self.lTokens, self.lBetween)

    def _analyze(self, lToi):
        for oToi in lToi:

            fStartLine = rules_utils.token_list_is_the_beginning_of_a_line(oToi.get_tokens())

            myToi = oToi.extract_tokens(1, 3)

            iLine, lTokens = utils.get_toi_parameters(myToi)
            dAction = {}

            if not fStartLine:
                check_spaces_on_left_side(lTokens, dAction, self.spaces_before)

            check_spaces_on_right_side(lTokens, dAction, self.spaces_after)

            if violations_found(dAction):
                sSolution = create_solution_text(dAction, self.spaces_before, self.spaces_after, lTokens)
                oViolation = violation.New(iLine, myToi, sSolution)
                oViolation.set_action(dAction)
                self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        fix_left_violations(self, dAction, lTokens)
        fix_right_violations(self, dAction, lTokens)
        oViolation.set_tokens(lTokens)


def fix_left_violations(self, dAction, lTokens):
    if not left_action_exists(dAction):
        return
    if dAction['left']['action'] == 'adjust':
        lTokens[0].set_value(' '*self.spaces_before)
    elif dAction['left']['action'] == 'remove':
        lTokens.pop(0)
    else:
        rules_utils.insert_whitespace(lTokens, self.spaces_before)


def fix_right_violations(self, dAction, lTokens):
    if not right_action_exists(dAction):
        return
    if dAction['right']['action'] == 'adjust':
        lTokens[-1].set_value(' '*self.spaces_after)
    else:
        rules_utils.insert_whitespace(lTokens, len(lTokens) - self.spaces_after)


def right_action_exists(dAction):
    if 'right' in list(dAction.keys()):
        return True
    return False


def left_action_exists(dAction):
    if 'left' in list(dAction.keys()):
        return True
    return False


def create_solution_text(dAction, iNumSpacesBefore, iNumSpacesAfter, lTokens):
    sReturn = ''
    sReturn += create_left_solution(dAction, iNumSpacesBefore, lTokens)
    sReturn += create_right_solution(dAction, iNumSpacesAfter, lTokens)
    return sReturn


def create_left_solution(dAction, iNumSpaces, lTokens):
    sReturn = ''
    if left_action_exists(dAction):
        sReturn = create_solution(dAction, 'left', iNumSpaces, lTokens)
    return sReturn


def create_right_solution(dAction, iNumSpaces, lTokens):
    sReturn = ''
    if right_action_exists(dAction):
        sReturn = create_solution(dAction, 'right', iNumSpaces, lTokens)
    return sReturn


def create_solution(dAction, sKey, iNumSpaces, lTokens):
    sSide = dAction[sKey]['side']
    sTokenValue = lTokens[1].get_value()
    if dAction[sKey]['action'] == 'adjust':
        sReturn = f'Change number of spaces {sSide} *{sTokenValue}* to {iNumSpaces}.  '
    elif dAction[sKey]['action'] == 'remove':
        sReturn = f'Remove all space(s) {sSide} *{sTokenValue}*. '
    else:
        sReturn = f'Add {iNumSpaces} space(s) {sSide} *{sTokenValue}*.  '
    return sReturn.strip()


def check_spaces_on_left_side(lTokens, dAction, iSpaces):
    check_for_adjustment_of_existing_whitespace(lTokens, dAction, iSpaces)
    check_for_removal_of_existing_whitespace(lTokens, dAction, iSpaces)
    check_for_insertion_of_missing_whitespace(lTokens, dAction, iSpaces)


def check_for_adjustment_of_existing_whitespace(lTokens, dAction, iSpaces):
    oLeft = lTokens[0]
    if isinstance(oLeft, parser.whitespace) and iSpaces > 0:
        set_adjust_action('left', oLeft, dAction, iSpaces)


def check_for_removal_of_existing_whitespace(lTokens, dAction, iSpaces):
    oLeft = lTokens[0]
    if isinstance(oLeft, parser.whitespace) and iSpaces == 0:
        set_remove_action('left', dAction)


def check_for_insertion_of_missing_whitespace(lTokens, dAction, iSpaces):
    oLeft = lTokens[0]
    if not isinstance(oLeft, parser.whitespace) and iSpaces > 0:
        set_insert_action('left', dAction)


def check_spaces_on_right_side(lTokens, dAction, iSpaces):
    oRight = lTokens[-1]
    if isinstance(oRight, parser.whitespace):
        set_adjust_action('right', oRight, dAction, iSpaces)
    else:
        set_insert_action('right', dAction)


def set_adjust_action(sSide, oToken, dAction, iSpaces):
    if iSpaces != len(oToken.get_value()):
        dAction[sSide] = {}
        dAction[sSide]['action'] = 'adjust'
        set_side_of_action(sSide, dAction)


def set_remove_action(sSide, dAction):
    dAction[sSide] = {}
    dAction[sSide]['action'] = 'remove'
    set_side_of_action(sSide, dAction)


def set_insert_action(sSide, dAction):
    dAction[sSide] = {}
    dAction[sSide]['action'] = 'insert'
    set_side_of_action(sSide, dAction)


def set_side_of_action(sSide, dAction):
    if sSide == 'right':
        dAction[sSide]['side'] = 'after'
    else:
        dAction[sSide]['side'] = 'before'


def violations_found(dAction):
    if len(list(dAction.keys())) > 0:
        return True
    return False





from vsg.rules import move_token_right_to_next_non_whitespace_token as Rule

from vsg import token

lTokens = []
lTokens.append(token.process_statement.process_label)


class rule_037(Rule):
    '''
    This rule checks a label and the colon are on the same line.

    **Violation**

    .. code-block:: vhdl

       label
       :

    **Fix**

    .. code-block:: vhdl

       label :
    '''

    def __init__(self):
        Rule.__init__(self, 'process', '037', lTokens)
        self.subphase = 3




from vsg.rules import experiment as Rule

from vsg.rules import option
from vsg import token
from vsg.rules import analysis
from vsg.rules import fix

lTokenPairs = []
lTokenPairs.append([token.concurrent_selected_signal_assignment.with_keyword, token.concurrent_selected_signal_assignment.semicolon])
lTokenPairs.append([token.selected_force_assignment.with_keyword, token.selected_force_assignment.semicolon])
lTokenPairs.append([token.selected_variable_assignment.with_keyword, token.selected_variable_assignment.semicolon])
lTokenPairs.append([token.selected_waveform_assignment.with_keyword, token.selected_waveform_assignment.semicolon])


class rule_001(Rule):
    '''
    This rule checks the structure of selected assignments.

    |configuring_selected_assignment_structure_rules_link|

    **Violation**

    .. code-block:: vhdl

       with mux_sel select addr := "0000"when 0, "0001" when 1, "1111"  when others;

    **Fix**

    .. code-block:: vhdl

       with mux_sel select
         addr := "0000" when 0,
                 "0001" when 1,
                 "1111" when others;
    '''

    def __init__(self):
        Rule.__init__(self, 'selected_assignment', '001')
        self.lTokenPairs = lTokenPairs
        self.add_option(newLineAfterWithKeywordOption())
        self.add_option(newLineBeforeSelectKeywordOption())
        self.add_option(newLineAfterSelectKeywordOption())
        self.add_option(newLineBeforeAssignmentOption())
        self.add_option(newLineAfterAssignmentOption())
        self.add_option(newLineBeforeWhenKeywordOption())
        self.add_option(newLineAfterWhenKeywordOption())
        self.add_option(newLineBeforeCommaOption())
        self.add_option(newLineAfterCommaOption())
        self.add_option(newLineBeforeSemicolonOption())

        self.add_option(singleLineWithExpressionOption())


def newLineAfterWithKeywordOption():
    oOption = option.New('new_line_after_with_keyword')
    oOption.value = 'no'
    oOption.analyze_function = analysis.check_for_carriage_return_after_token

    lOptions = []
    lOptions.append(token.concurrent_selected_signal_assignment.with_keyword)
    lOptions.append(token.selected_force_assignment.with_keyword)
    lOptions.append(token.selected_variable_assignment.with_keyword)
    lOptions.append(token.selected_waveform_assignment.with_keyword)

    oOption.analysis_options = lOptions

    return oOption


def newLineBeforeSelectKeywordOption():

    oOption = option.New('new_line_before_select_keyword')
    oOption.value = 'no'
    oOption.analyze_function = analysis.check_for_carriage_return_before_token

    lOptions = []
    lOptions.append(token.concurrent_selected_signal_assignment.select_keyword)
    lOptions.append(token.selected_force_assignment.select_keyword)
    lOptions.append(token.selected_variable_assignment.select_keyword)
    lOptions.append(token.selected_waveform_assignment.select_keyword)

    oOption.analysis_options = lOptions
    return oOption


def newLineAfterSelectKeywordOption():

    oOption = option.New('new_line_after_select_keyword')
    oOption.value = 'yes'
    oOption.analyze_function = analysis.check_for_carriage_return_after_token

    lOptions = []
    lOptions.append(token.concurrent_selected_signal_assignment.select_keyword)
    lOptions.append(token.selected_force_assignment.select_keyword)
    lOptions.append(token.selected_variable_assignment.select_keyword)
    lOptions.append(token.selected_waveform_assignment.select_keyword)

    oOption.analysis_options = lOptions
    return oOption


def newLineBeforeAssignmentOption():

    oOption = option.New('new_line_before_assignment')
    oOption.value = 'no'
    oOption.analyze_function = analysis.check_for_carriage_return_before_token

    lOptions = []
    lOptions.append(token.concurrent_selected_signal_assignment.assignment)
    lOptions.append(token.selected_force_assignment.assignment)
    lOptions.append(token.selected_variable_assignment.assignment)
    lOptions.append(token.selected_waveform_assignment.assignment)

    oOption.analysis_options = lOptions
    return oOption


def newLineAfterAssignmentOption():

    oOption = option.New('new_line_after_assignment')
    oOption.value = 'no'
    oOption.analyze_function = analysis.check_for_carriage_return_after_token

    lOptions = []
    lOptions.append(token.concurrent_selected_signal_assignment.assignment)
    lOptions.append(token.selected_force_assignment.assignment)
    lOptions.append(token.selected_variable_assignment.assignment)
    lOptions.append(token.selected_waveform_assignment.assignment)

    oOption.analysis_options = lOptions
    return oOption

def newLineBeforeWhenKeywordOption():

    oOption = option.New('new_line_before_when_keyword')
    oOption.value = 'no'
    oOption.analyze_function = analysis.check_for_carriage_return_before_token

    lOptions = []
    lOptions.append(token.selected_expressions.when_keyword)
    lOptions.append(token.selected_waveforms.when_keyword)

    oOption.analysis_options = lOptions
    return oOption

def newLineAfterWhenKeywordOption():

    oOption = option.New('new_line_after_when_keyword')
    oOption.value = 'no'
    oOption.analyze_function = analysis.check_for_carriage_return_after_token

    lOptions = []
    lOptions.append(token.selected_expressions.when_keyword)
    lOptions.append(token.selected_waveforms.when_keyword)

    oOption.analysis_options = lOptions
    return oOption

def newLineBeforeCommaOption():

    oOption = option.New('new_line_before_comma')
    oOption.value = 'no'
    oOption.analyze_function = analysis.check_for_carriage_return_before_token

    lOptions = []
    lOptions.append(token.selected_expressions.comma)
    lOptions.append(token.selected_waveforms.comma)

    oOption.analysis_options = lOptions
    return oOption

def newLineAfterCommaOption():

    oOption = option.New('new_line_after_comma')
    oOption.value = 'yes'
    oOption.analyze_function = analysis.check_for_carriage_return_after_token

    lOptions = []
    lOptions.append(token.selected_expressions.comma)
    lOptions.append(token.selected_waveforms.comma)

    oOption.analysis_options = lOptions
    return oOption

def newLineBeforeSemicolonOption():

    oOption = option.New('new_line_before_semicolon')
    oOption.value = 'no'
    oOption.analyze_function = analysis.check_for_carriage_return_before_token

    lOptions = []
    lOptions.append(token.concurrent_selected_signal_assignment.semicolon)
    lOptions.append(token.selected_force_assignment.semicolon)
    lOptions.append(token.selected_variable_assignment.semicolon)
    lOptions.append(token.selected_waveform_assignment.semicolon)

    oOption.analysis_options = lOptions
    return oOption

def singleLineWithExpressionOption():

    oOption = option.New('single_line_with_expression')
    oOption.value = 'ignore'
    oOption.analyze_function = analysis.check_for_carriage_returns_between_tokens_ignoring_leading_and_trailing_whitespace

    lOptions = []
    lOptions.append([token.concurrent_selected_signal_assignment.with_keyword, token.concurrent_selected_signal_assignment.select_keyword])
    lOptions.append([token.selected_force_assignment.with_keyword, token.selected_force_assignment.select_keyword])
    lOptions.append([token.selected_variable_assignment.with_keyword, token.selected_variable_assignment.select_keyword])
    lOptions.append([token.selected_waveform_assignment.with_keyword, token.selected_waveform_assignment.select_keyword])

    oOption.analysis_options = lOptions
    return oOption




from vsg import parser


class label(parser.label):
    '''
    unique_id = assertion_statement : label
    '''

    def __init__(self, sString):
        parser.label.__init__(self, sString)


class label_colon(parser.label_colon):
    '''
    unique_id = assertion_statement : label_colon
    '''

    def __init__(self):
        parser.label_colon.__init__(self)


class semicolon(parser.semicolon):
    '''
    unique_id = assertion_statement : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)




from vsg.rules import align_tokens_in_region_between_tokens_unless_between_tokens

from vsg import token

lAlign = []
lAlign.append(token.file_declaration.colon)
lAlign.append(token.signal_declaration.colon)
lAlign.append(token.constant_declaration.colon)
lAlign.append(token.variable_declaration.colon)
lAlign.append(token.alias_declaration.colon)
lAlign.append(token.alias_declaration.is_keyword)

lUnless = []
lUnless.append([token.subprogram_body.is_keyword,token.subprogram_body.begin_keyword])


class rule_026(align_tokens_in_region_between_tokens_unless_between_tokens):
    '''
    This rule checks the colons are in the same column for all declarations in the architecture declarative part.

    |configuring_keyword_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

       architecture rtl of my_entity is

         signal   wr_en : std_logic;
         signal   rd_en   : std_logic;
         constant c_period : time;

       begin

    **Fix**

    .. code-block:: vhdl

       architecture rtl of my_entity is

         signal   wr_en    : std_logic;
         signal   rd_en    : std_logic;
         constant c_period : time;

       begin
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens_unless_between_tokens.__init__(self, 'architecture', '026', lAlign, token.architecture_body.is_keyword, token.architecture_body.begin_keyword, lUnless)
        self.solution = 'Align identifer.'
        self.subphase = 3




from vsg.token import assertion_statement as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import assertion


def detect(iToken, lObjects):
    '''
    assertion_statement ::=
        [ label : ] assertion ;
    '''
    if utils.keyword_found('assert', iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.tokenize_label(iToken, lObjects, token.label, token.label_colon)

    iCurrent = assertion.classify(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.token import file_declaration as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import file_open_information
from vsg.vhdlFile.classify import identifier_list
from vsg.vhdlFile.classify import subtype_indication


def detect(iToken, lObjects):
    '''
    file_declaration ::=
        file identifier_list : subtype_indication [ file_open_information ] ;
    '''

    if utils.is_next_token('file', iToken, lObjects):
        return classify(iToken, lObjects)

    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('file', token.file_keyword, iToken, lObjects)
    iCurrent = identifier_list.classify_until([':'], iCurrent, lObjects, token.identifier)
    iCurrent = utils.assign_next_token_required(':', token.colon, iCurrent, lObjects)

    iCurrent= subtype_indication.classify(iCurrent, lObjects)

    iCurrent = file_open_information.detect(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.rules import blank_line_below_line_ending_with_token as Rule

from vsg import token

lTokens = []
lTokens.append(token.port_map_aspect.open_parenthesis)


class rule_200(Rule):
    '''
    This rule checks for a blank line below the open parenthesis in a port map.

    |configuring_blank_lines_link|

    **Violation**

    .. code-block:: vhdl

      port map (

        WR_EN => w_wr_en,
        RD_EN => w_rd_en,
        OVERFLOW => w_overflow
      );

    **Fix**

    .. code-block:: vhdl

      port map (
        WR_EN => w_wr_en,
        RD_EN => w_rd_en,
        OVERFLOW => w_overflow
      );
    '''

    def __init__(self):
        Rule.__init__(self, 'port_map', '200', lTokens)
        self.style = 'no_blank_line'




from vsg import parser
from vsg import token

from vsg.rules.whitespace_after_token import Rule

lTokens = []
lTokens.append(token.if_statement.elsif_keyword)


class rule_005(Rule):
    '''
    This rule checks for a single space after the **elsif** keyword.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

      elsif(c = '1') then

      elsif   (c = '1') then

      elsif    b = '0' then

    **Fix**

    .. code-block:: vhdl

      elsif (c = '1') then

      elsif (c = '1') then

      elsif b = '0' then
    '''
    def __init__(self):
        Rule.__init__(self, 'if', '005', lTokens)




from vsg import parser


class array_keyword(parser.keyword):
    '''
    unique_id = unbounded_array_definition : array_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class open_parenthesis(parser.open_parenthesis):
    '''
    unique_id = unbounded_array_definition : open_parenthesis
    '''

    def __init__(self, sString='('):
        parser.open_parenthesis.__init__(self)


class comma(parser.comma):
    '''
    unique_id = unbounded_array_definition : comma
    '''

    def __init__(self, sString=','):
        parser.comma.__init__(self)


class close_parenthesis(parser.close_parenthesis):
    '''
    unique_id = unbounded_array_definition : close_parenthesis
    '''

    def __init__(self, sString=')'):
        parser.close_parenthesis.__init__(self)


class of_keyword(parser.keyword):
    '''
    unique_id = unbounded_array_definition : of_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.package_body.package_keyword)


class rule_300(token_indent):
    '''
    This rule checks the indent of the package body keyword.

    **Violation**

    .. code-block:: vhdl

       library ieee;

         package body FIFO_PKG is

    **Fix**

    .. code-block:: vhdl

       library ieee;

       package body FIFO_PKG is
    '''

    def __init__(self):
        token_indent.__init__(self, 'package_body', '300', lTokens)




from vsg.rules import create_violation


class New():

    def __init__(self, sName):
        self.name = sName 
        self.value = None
        self.analyze_function = None
        self.analysis_options = None

    def analyze(self, oToi):
        return self.analyze_function(self, oToi)




from vsg import parser


class alias_keyword(parser.keyword):
    '''
    unique_id = alias_declaration : alias_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class alias_designator(parser.designator):
    '''
    unique_id = alias_declaration : alias_designator
    '''

    def __init__(self, sString):
        parser.designator.__init__(self, sString)


class colon(parser.colon):
    '''
    unique_id = alias_declaration : colon
    '''

    def __init__(self, sString=':'):
        parser.colon.__init__(self)


class is_keyword(parser.keyword):
    '''
    unique_id = alias_declaration : is_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class semicolon(parser.semicolon):
    '''
    unique_id = alias_declaration : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)


class name(parser.name):
    '''
    unique_id = alias_declaration : name
    '''

    def __init__(self, sString):
        parser.name.__init__(self, sString)




from vsg.rules import insert_token_left_of_token_if_it_does_not_exist_between_tokens_using_value_from_token

from vsg.token import package_body as token

oInsertToken = token.end_package_simple_name

oLeftToken = token.semicolon

oStartToken = token.end_keyword
oEndToken = token.semicolon

oValueToken = token.package_simple_name


class rule_003(insert_token_left_of_token_if_it_does_not_exist_between_tokens_using_value_from_token):
    '''
    This rule checks the package name exists in the closing of the package body declaration.

    |configuring_optional_items_link|

    **Violation**

    .. code-block:: vhdl

       end package body;

    **Fix**

    .. code-block:: vhdl

       end package body fifo_pkg;
    '''

    def __init__(self):
        insert_token_left_of_token_if_it_does_not_exist_between_tokens_using_value_from_token.__init__(self, 'package_body', '003', oInsertToken, oLeftToken, oStartToken, oEndToken, oValueToken)
        self.solution = 'Add package name.'




from vsg.rules import multiline_conditional_alignment as Rule

from vsg import token

lTokenPairs = []
lTokenPairs.append([token.conditional_waveform_assignment.assignment, token.conditional_waveform_assignment.semicolon])


class rule_401(Rule):
    '''
    This rule checks alignment of multiline sequential conditional signal assignments.

    |configuring_conditional_multiline_indent_rules_link|

    **Violation**

    .. code-block:: vhdl

       wr_en <= '0' when q_wr_en = '1' else
            '1';

       w_foo <= I_FOO when ((I_BAR = '1') and
                (I_CRUFT = '1')) else
                '0';

    **Fix**

    .. code-block:: vhdl

       wr_en <= '0' when q_wr_en = '1' else
                '1';

       w_foo <= I_FOO when ((I_BAR = '1') and
                            (I_CRUFT = '1')) else
                '0';
    '''

    def __init__(self):
        Rule.__init__(self, 'sequential', '401', lTokenPairs)




from vsg import parser


class entity_tag(parser.identifier):
    '''
    unique_id = entity_designator : entity_tag
    '''

    def __init__(self, sString):
        parser.identifier.__init__(self, sString)




from vsg import parser
from vsg import token

from vsg.rules import consistent_case_of_tokens_from_between_tokens_applied_to_region

lTokens = []
lTokens.append(token.interface_unknown_declaration.identifier)
lTokens.append(token.interface_constant_declaration.identifier)
lTokens.append(token.interface_variable_declaration.identifier)
lTokens.append(token.interface_signal_declaration.identifier)

lIgnore = []
lIgnore.append(parser.whitespace)
lIgnore.append(parser.carriage_return)
lIgnore.append(parser.blank_line)
lIgnore.append(token.identifier.identifier)

oStartToken = token.generic_clause.open_parenthesis
oEndToken = token.generic_clause.close_parenthesis

oRegionStart = token.port_clause.open_parenthesis
oRegionEnd = token.port_clause.close_parenthesis


class rule_600(consistent_case_of_tokens_from_between_tokens_applied_to_region):
    '''
    This rule checks for consistent capitalization of generic names in entity declarations.

    **Violation**

    .. code-block:: vhdl

       entity FIFO is
         generic (
           G_WIDTH : natural := 16
         );
         port (
           I_DATA : std_logic_vector(g_width - 1 downto 0);
           O_DATA : std_logic_vector(g_width - 1 downto 0)
         );
       end entity fifo;

    **Fix**

    .. code-block:: vhdl

       entity FIFO is
         generic (
           G_WIDTH : natural := 16
         );
         port (
           I_DATA : std_logic_vector(G_WIDTH - 1 downto 0);
           O_DATA : std_logic_vector(G_WIDTH - 1 downto 0)
         );
       end entity fifo;
    '''

    def __init__(self):
        consistent_case_of_tokens_from_between_tokens_applied_to_region.__init__(self, 'entity', '600', lTokens, oStartToken, oEndToken, oRegionStart, oRegionEnd, lIgnore)




from vsg.rules import token_case_in_range_bounded_by_tokens_with_prefix_suffix

from vsg import token

lTokens = []
lTokens.append(token.subprogram_body.designator)

oStartToken = token.procedure_specification.procedure_keyword
oEndToken = token.subprogram_body.semicolon


class rule_506(token_case_in_range_bounded_by_tokens_with_prefix_suffix):
    '''
    This rule checks the procedure designator has proper case on the end procedure declaration.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       end procedure AVERAGE_SAMPLES;

    **Fix**

    .. code-block:: vhdl

       end procedure average_samples;
    '''

    def __init__(self):
        token_case_in_range_bounded_by_tokens_with_prefix_suffix.__init__(self, 'procedure', '506', lTokens, oStartToken, oEndToken)
        self.groups.append('case::name')




from vsg.rules import blank_line_below_line_ending_with_token

from vsg import token

lTokens = []
lTokens.append(token.component_instantiation_statement.semicolon)


class rule_019(blank_line_below_line_ending_with_token):
    '''
    This rule checks for a blank line below the end of the instantiation declaration.

    |configuring_blank_lines_link|

    **Violation**

    .. code-block:: vhdl

       U_FIFO : FIFO
         port map (
           WR_EN    => wr_en,
           RD_EN    => rd_en,
           OVERFLOW => overflow
         );
       U_RAM : RAM

    **Fix**

    .. code-block:: vhdl

       U_FIFO : FIFO
         port map (
           WR_EN    => wr_en,
           RD_EN    => rd_en,
           OVERFLOW => overflow
         );

       U_RAM : RAM
    '''

    def __init__(self):
        blank_line_below_line_ending_with_token.__init__(self, 'instantiation', '019', lTokens)




from vsg.token import subprogram_instantiation_declaration as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import signature
from vsg.vhdlFile.classify import subprogram_kind
from vsg.vhdlFile.classify import generic_map_aspect


def detect(iToken, lObjects):
    '''
    subprogram_instantiation_declaration ::=
        subprogram_kind identifier is new uninstantiated_subprogram_name [ signature ]
            [ generic_map_aspect ] ;
    '''

    if subprogram_kind.detect(iToken, lObjects):
        if utils.find_in_next_n_tokens('is', 3, iToken, lObjects):
            if utils.find_in_next_n_tokens('new', 4, iToken, lObjects):
                return classify(iToken, lObjects)
        else:
            return iToken
    return iToken


def classify(iToken, lObjects):

    iCurrent = subprogram_kind.classify(iToken, lObjects)

    iCurrent = utils.assign_next_token(token.identifier, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('is', token.is_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('new', token.new_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token(token.uninstantiated_subprogram_name, iCurrent, lObjects)

    iCurrent = signature.detect(iCurrent, lObjects)

    iCurrent = generic_map_aspect.detect(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg import parser
from vsg import violation

from vsg.rules import utils as rules_utils
from vsg.rule_group import structure
from vsg.vhdlFile import utils


class move_token_to_the_right_of_several_possible_tokens_if_it_exists_between_tokens(structure.Rule):
    '''
    Moves one token next to several possible tokens if it exists.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    oToken : token type
       The token to move.

    lTokens : list of token type
       The token types to move oToken to.
       This is an ordered list with the left item containing the token furthest way from oToken.
    '''

    def __init__(self, name, identifier, oMoveToken, lAnchorTokens, oStartToken, oEndToken, bInsertWhitespace=False):
        structure.Rule.__init__(self, name, identifier)
        self.oMoveToken = oMoveToken
        self.lAnchorTokens = lAnchorTokens
        self.oStartToken = oStartToken
        self.oEndToken = oEndToken
        self.bInsertWhitespace = bInsertWhitespace
        self.configuration_documentation_link = None

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_bounded_by(self.oStartToken, self.oEndToken, bIncludeTillEndOfLine=True)

    def _analyze(self, lToi):
        for oToi in lToi:
            iLine, lTokens = utils.get_toi_parameters(oToi)
            if not utils.does_token_type_exist_in_list_of_tokens(self.oMoveToken, lTokens):
                continue
            dAction = {}
            bPassing = False
            for iToken, oToken in enumerate(lTokens):
                iLine = utils.increment_line_number(iLine, oToken)
                for oAnchorToken in self.lAnchorTokens:
                    if isinstance(oToken, oAnchorToken):
                        dAction['insert'] = iToken + 1
                        sAnchorToken = oToken.get_value()
                        iAnchorLine = iLine
                        if utils.are_next_consecutive_token_types([parser.whitespace, self.oMoveToken], iToken + 1, lTokens):
                            bPassing = True
                            break
                        elif isinstance(lTokens[iToken + 1], self.oMoveToken):
                            bPassing = True
                            break
                if isinstance(oToken, self.oMoveToken):
                    iAnchorLine = iLine
                    dAction['move_index'] = iToken
                    sSolution = 'Move "' + oToken.get_value() + '" on line ' + str(iLine) + ' to the right of "' + sAnchorToken + '" on line ' + str(iAnchorLine)
                if bPassing:
                    break
            else:
                oViolation = violation.New(iAnchorLine, oToi, sSolution)
                oViolation.set_action(dAction)
                oViolation.set_remap()
                self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()

        oToken = lTokens.pop(dAction['move_index'])
        rules_utils.insert_token(lTokens, dAction['insert'], oToken)
        if self.bInsertWhitespace:
            rules_utils.insert_whitespace(lTokens, dAction['insert'])
        lTokens = utils.remove_consecutive_whitespace_tokens(lTokens)
        lTokens = utils.fix_blank_lines(lTokens)
        oViolation.set_tokens(lTokens)




from vsg.rules import blank_line_below_line_ending_with_token

from vsg import token

lTokens = []
lTokens.append(token.case_generate_statement.semicolon)
lTokens.append(token.for_generate_statement.semicolon)
lTokens.append(token.if_generate_statement.semicolon)


class rule_003(blank_line_below_line_ending_with_token):
    '''
    This rule checks for a blank line below the **end generate** keywords.

    |configuring_blank_lines_link|

    **Violation**

    .. code-block:: vhdl

       end generate ram_array;
       wr_en <= '1';

    **Fix**

    .. code-block:: vhdl

       end generate ram_array;

       wr_en <= '1';
    '''

    def __init__(self):
        blank_line_below_line_ending_with_token.__init__(self, 'generate', '003', lTokens)




from vsg.vhdlFile.classify import interface_object_declaration
from vsg.vhdlFile.classify import interface_package_declaration
from vsg.vhdlFile.classify import interface_subprogram_declaration
from vsg.vhdlFile.classify import interface_type_declaration


def detect(iToken, lObjects):
    '''
    interface_declaration ::=
        interface_object_declaration
      | interface_type_declaration
      | interface_subprogram_declaration
      | interface_package_declaration
    '''

    iCurrent = interface_object_declaration.detect(iToken, lObjects)
    if iCurrent != iToken:
        return iCurrent

    iCurrent = interface_type_declaration.detect(iToken, lObjects)
    if iCurrent != iToken:
        return iCurrent

    iCurrent = interface_subprogram_declaration.detect(iToken, lObjects)
    if iCurrent != iToken:
        return iCurrent

    iCurrent = interface_package_declaration.detect(iToken, lObjects)
    if iCurrent != iToken:
        return iCurrent

    return iToken




from vsg.rules import multiline_structure as Rule

from vsg import token

lTokenPairs = []
lTokenPairs.append([token.constant_declaration.constant_keyword, token.constant_declaration.semicolon])


class rule_016(Rule):
    '''
    This rule checks the structure of multiline constants that contain arrays.

    |configuring_array_multiline_structure_rules_link|

    .. NOTE:: The indenting of multiline array constants is handled by the rule `constant_012 <constant_rules.html#constant-012>`_.

    **Violation**

    .. code-block:: vhdl

       constant rom : romq_type := (0, 65535, 32768);

    **Fix**

    .. code-block:: vhdl

       constant rom : romq_type :=
       (
         0,
         65535,
         32768
       );
    '''

    def __init__(self):
        Rule.__init__(self, 'constant', '016', lTokenPairs)
        self.assignment_operator = token.constant_declaration.assignment_operator
        self.semicolon = token.constant_declaration.semicolon
        self.phase = 5




from vsg.rules import insert_token_right_of_token_if_it_does_not_exist_between_tokens_using_value_from_token as Rule

from vsg import token

oInsertToken = token.subprogram_kind.function_keyword

oAnchorToken = token.subprogram_body.end_keyword

oLeftToken = token.function_specification.function_keyword
oRightToken = token.subprogram_body.semicolon

oValueToken = token.function_specification.function_keyword


class rule_018(Rule):
    '''
    This rule checks the function keyword exist in the closing of the function specification.

    |configuring_optional_items_link|

    **Violation**

    .. code-block:: vhdl

       function func return integer is

       end func;

    **Fix**

    .. code-block:: vhdl

       function func return integer is

       end function func;
    '''

    def __init__(self):
        Rule.__init__(self, 'function', '018', oInsertToken, oAnchorToken, oLeftToken, oRightToken, oValueToken)
        self.solution = 'function keyword'
        self.groups.append('structure::optional')
        self.filter_tokens.append(token.subprogram_declaration.semicolon)




from vsg import token

from vsg.rules import n_spaces_before_and_after_tokens

lTokens = []
lTokens.append(token.adding_operator.concat)


class rule_010(n_spaces_before_and_after_tokens):
    '''
    This rule checks for spaces before and after the concate (&) operator.

    **Violation**

    .. code-block:: vhdl

       a <= b&c;

    **Fix**

    .. code-block:: vhdl

       a <= b & c;
    '''

    def __init__(self):
        n_spaces_before_and_after_tokens.__init__(self, 'whitespace', '010', 1, lTokens)
        self.solution = 'Ensure a single space before and after concat operator.'
        self.configuration_documentation_link = None




from vsg.rules import move_token_next_to_another_token

from vsg.token import context_declaration as token


class rule_006(move_token_next_to_another_token):
    '''
    This rule checks the **is** keyword is on the same line as the context identifier.

    **Violation**

    .. code-block:: vhdl

       context c1
         is

    **Fix**

    .. code-block:: vhdl

       context c1 is
    '''

    def __init__(self):
        move_token_next_to_another_token.__init__(self, 'context', '006', token.identifier, token.is_keyword)
        self.subphase = 2
        self.solution = 'Move *is* keyword next to context identifier'




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.report_statement.report_keyword)
lTokens.append(token.report_statement.severity_keyword)


class rule_300(token_indent):
    '''
    This rule checks indent of multiline report statements.

    **Violation**

    .. code-block:: vhdl

        report "FIFO width is limited to 16 bits."
                severity FAILURE;

    **Fix**

    .. code-block:: vhdl

        report "FIFO width is limited to 16 bits."
          severity FAILURE;
    '''

    def __init__(self):
        token_indent.__init__(self, 'report_statement', '300', lTokens)




from vsg.rules import token_suffix_between_tokens

from vsg import token

lTokens = []
lTokens.append(token.association_element.formal_part)

lStart = token.generic_map_aspect.open_parenthesis
lEnd = token.generic_map_aspect.close_parenthesis


class rule_600(token_suffix_between_tokens):
    '''
    This rule checks for valid suffixes on generic identifiers in generic maps
    The default generic suffix is *\_g*.

    |configuring_prefix_and_suffix_rules_link|

    **Violation**

    .. code-block:: vhdl

       generic map
       (
         WIDTH => 32,
         DEPTH => 512
       )

    **Fix**

    .. code-block:: vhdl

       generic map
       (
         WIDTH_G => 32,
         DEPTH_G => 512
       )
    '''

    def __init__(self):
        token_suffix_between_tokens.__init__(self, 'generic_map', '600', lTokens, lStart, lEnd)
        self.suffixes = ['_g']




from vsg.rules import align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens

from vsg import parser
from vsg import token

lAlign = []
lAlign.append(parser.comment)

lSkip = []
lSkip.append(parser.comment)


class rule_029(align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens):
    '''
    This rule checks for alignment of inline comments in an instantiation.

    Following extra configurations are supported:

    * :code:`separate_generic_port_alignment`.

    |configuring_keyword_alignment_rules_link|
    **Violation**

    **Violation**

    .. code-block:: vhdl

           wr_en    => write_enable,        -- Wrte enable
           rd_en    => read_enable,    -- Read enable
           overflow => overflow,         -- FIFO has overflowed

    **Fix**

    .. code-block:: vhdl

           wr_en    => write_enable, -- Wrte enable
           rd_en    => read_enable,  -- Read enable
           overflow => overflow,     -- FIFO has overflowed
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens.__init__(self, 'instantiation', '029', lAlign, token.component_instantiation_statement.instantiation_label, token.component_instantiation_statement.semicolon, lSkip)
        self.solution = 'Align comment.'
        self.subphase = 3




from vsg import parser


class record_keyword(parser.keyword):
    '''
    unique_id = record_type_definition : record_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_keyword(parser.keyword):
    '''
    unique_id = record_type_definition : end_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_record_keyword(parser.keyword):
    '''
    unique_id = record_type_definition : end_record_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class record_type_simple_name(parser.simple_name):
    '''
    unique_id = record_type_definition : record_type_simple_name
    '''

    def __init__(self, sString):
        parser.simple_name.__init__(self, sString)




from vsg import parser


class open_parenthesis(parser.open_parenthesis):
    '''
    unique_id = enumeration_type_definition : open_parenthesis
    '''

    def __init__(self, sString='('):
        parser.open_parenthesis.__init__(self)


class enumeration_literal(parser.item):
    '''
    unique_id = enumeration_type_definition : enumeration_literal
    '''

    def __init__(self, sString):
        parser.item.__init__(self, sString)


class comma(parser.comma):
    '''
    unique_id = enumeration_type_definition : comma
    '''

    def __init__(self, sString=','):
        parser.comma.__init__(self)


class close_parenthesis(parser.close_parenthesis):
    '''
    unique_id = enumeration_type_definition : close_parenthesis
    '''

    def __init__(self, sString=')'):
        parser.close_parenthesis.__init__(self)





from vsg import token
from vsg import violation
from vsg.rule_group import structure


class rule_034(structure.Rule):
    '''
    This rule checks for component versus direct instantiations.

    |configuring_type_of_instantiations_link|

    component instantiation
    ^^^^^^^^^^^^^^^^^^^^^^^

    .. NOTE:: This is the default configuration

    **Violation**

    .. code-block:: vhdl

       U_FIFO : entity fifo_dsn.FIFO(RTL)


    entity instantiation
    ^^^^^^^^^^^^^^^^^^^^

    **Violation**

    .. code-block:: vhdl

       U_FIFO : component FIFO

       U_FIFO : FIFO
    '''

    def __init__(self):
        structure.Rule.__init__(self, name='instantiation', identifier='034')
        self.method = 'component'
        self.configuration.append('method')
        self.fixable = False
        self.configuration_documentation_link = 'configuring_type_of_instantiations_link'

    def _get_tokens_of_interest(self, oFile):
        if self.method == 'component':
            return oFile.get_tokens_matching_in_range_bounded_by_tokens([token.instantiated_unit.entity_keyword], token.component_instantiation_statement.instantiation_label, token.component_instantiation_statement.semicolon)
        else:
            return oFile.get_tokens_matching_in_range_bounded_by_tokens([token.instantiated_unit.component_name], token.component_instantiation_statement.instantiation_label, token.component_instantiation_statement.semicolon)

    def _analyze(self, lToi):
        for oToi in lToi:
            if self.method == 'component':
                sSolution = 'Change to component instantiation'
            else:
                sSolution = 'Change to entity instantiation'
            self.add_violation(violation.New(oToi.get_line_number(), oToi, sSolution))




from vsg.rules import insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment

from vsg.token import if_statement as token

lTokens = []
lTokens.append(token.then_keyword)


class rule_024(insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment):
    '''
    This rule checks for code after the **then** keyword.

    **Violation**

    .. code-block:: vhdl

       if (a = '1') then c <= '1';

    **Fix**

    .. code-block:: vhdl

       if (a = '1') then
         c <= '1';
    '''

    def __init__(self):
        insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment.__init__(self, 'if', '024', lTokens)
        self.solution = 'Move code after *then* keyword to the next line.'



#!/usr/bin/env python

import argparse
import sys

from . import vhdlFile

from vsg.tests import utils
from .CustomArgumentParser import CustomArgumentParser


def parse_command_line_arguments():
    '''Parses the command line arguments and returns them.'''

    parser = CustomArgumentParser(
      prog='VHDL Style Guide (VSG) Parser',
      description='''Outputs the results from parsing a VHDL file.''')

    parser.add_argument('-f', '--filename', help='File to print parser output')
    parser.add_argument('-w', '--whitespace', default=False, action='store_true', help='Include whitespace objects')

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)
    else:
        return parser.parse_args()


def main():
    '''Main routine of parser output'''

    fExitStatus = 0

    commandLineArguments = parse_command_line_arguments()

    sFileName = commandLineArguments.filename

    lLines = vhdlFile.utils.read_vhdlfile(sFileName)

    oVhdlFile = vhdlFile.vhdlFile(lLines[0])
    oVhdlFile.filename = sFileName

    utils.print_objects(oVhdlFile, not commandLineArguments.whitespace)

    sys.exit(fExitStatus)


if __name__ == '__parser__':
    main()




from vsg.rules import token_case_in_range_bounded_by_tokens

from vsg import token

lTokens = []
lTokens.append(token.interface_constant_declaration.identifier)
lTokens.append(token.interface_signal_declaration.identifier)
lTokens.append(token.interface_variable_declaration.identifier)
lTokens.append(token.interface_file_declaration.identifier)
lTokens.append(token.interface_unknown_declaration.identifier)

oStart = token.generic_clause.open_parenthesis
oEnd = token.generic_clause.close_parenthesis


class rule_007(token_case_in_range_bounded_by_tokens):
    '''
    This rule checks the generic names have proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       G_WIDTH : integer := 32;

    **Fix**

    .. code-block:: vhdl

       g_width : integer := 32;
    '''

    def __init__(self):
        token_case_in_range_bounded_by_tokens.__init__(self, 'generic', '007', lTokens, oStart, oEnd)
        self.configuration.append('prefix_exceptions')
        self.configuration.append('suffix_exceptions')
        self.configuration.append('case_exceptions')
        self.groups.append('case::name')




from vsg import parser

from vsg.vhdlFile.extract import utils


def get_blank_lines_above_line_starting_with_token(lTokens, lAllTokens, oTokenMap):

    lIndexes = utils.get_indexes_of_token_list(lTokens, oTokenMap)
    lIndexes = utils.filter_indexes_which_start_a_line(lIndexes, oTokenMap)
    return utils.get_all_blank_lines_above_indexes(lIndexes, lAllTokens, oTokenMap)




from vsg.rules import previous_line

from vsg.token import case_statement as token


class rule_007(previous_line):
    '''
    This rule checks for blank lines or comments above the **case** keyword.

    |configuring_previous_line_rules_link|

    The default style is :code:`no_code`.

    **Violation**

    .. code-block:: vhdl

       a <= '1';
       case data is


       -- This is a comment
       case data is

    **Fix**

    .. code-block:: vhdl

       a <= '1';

       case data is


       -- This is a comment
       case data is
    '''

    def __init__(self):
        previous_line.__init__(self, 'case', '007', [token.case_keyword])
        self.style = 'no_code'




from vsg.token import concurrent_selected_signal_assignment as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import delay_mechanism
from vsg.vhdlFile.classify import expression
from vsg.vhdlFile.classify import selected_waveforms


def detect(iToken, lObjects):
    '''
    concurrent_selected_signal_assignment ::=
        with expression select [ ? ]
            target <= [ guarded ] [ delay_mechanism ] selected_waveforms ;

    The key to detecting this is looking for the **with** keyword before the **select** keyword.
    '''
    if utils.find_in_next_n_tokens('with', 4, iToken, lObjects):
        if not utils.find_in_next_n_tokens('end', 1, iToken, lObjects):
            return True
    return False


def classify(iToken, lObjects):
    iCurrent = utils.assign_next_token_required('with', token.with_keyword, iToken, lObjects)

    iCurrent = expression.classify_until(['select'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('select', token.select_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if('?', token.question_mark, iCurrent, lObjects)

    iCurrent = utils.assign_tokens_until('<=', token.target, iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('<=', token.assignment, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if('guarded', token.guarded_keyword, iCurrent, lObjects)

    iCurrent = delay_mechanism.detect(iCurrent, lObjects)

    selected_waveforms.classify(iToken, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.rules import token_indent as Rule

from vsg import token

lTokens = []
lTokens.append(token.if_generate_statement.elsif_keyword)


class rule_300(Rule):
    '''
    This rule checks the indent of the *elsif* keyword.

    **Violation**

    .. code-block:: vhdl

       ram_array : if condition generate
          elsif condition generate
       end generate;

    **Fix**

    .. code-block:: vhdl

       ram_array : if condition generate
       elsif condition generate
       end generate;
    '''

    def __init__(self):
        Rule.__init__(self, 'if_generate_statement', '300', lTokens)




from vsg.token import package_instantiation_declaration as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import generic_map_aspect
from vsg.vhdlFile.classify import identifier


def detect(iToken, lObjects):
    '''
    package_instantiation_declaration ::=
        package identifier is new *uninstantiated_package*_name
            [ generic_map_aspect ] ;
    '''

    if utils.are_next_consecutive_tokens(['package', None, 'is', 'new'], iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('package', token.package_keyword, iToken, lObjects)

    iCurrent = identifier.classify(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('is', token.is_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('new', token.new_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token(token.uninstantiated_package_name, iCurrent, lObjects)

    iCurrent = generic_map_aspect.detect(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.vhdlFile.extract import tokens
from vsg.vhdlFile.extract import utils


def get_sequence_of_tokens_matching(lTokens, lAllTokens, oTokenMap, bIgnoreIfLineStart=False):
    lReturn = []

    lIndexes = get_token_indexes(lTokens, oTokenMap)

    for iIndex in lIndexes:
        iLine = oTokenMap.get_line_number_of_index(iIndex)
        if bIgnoreIfLineStart and utils.is_token_at_start_of_line(iIndex, oTokenMap):
            continue
        for iToken, oToken in enumerate(lTokens):
            if not isinstance(lAllTokens[iToken + iIndex], oToken):
                break
        else:
            lReturn.append(tokens.New(iIndex, iLine, lAllTokens[iIndex:iIndex + len(lTokens)]))

    return lReturn


def get_token_indexes(lTokens, oTokenMap):
    lIndexes = oTokenMap.get_token_indexes(lTokens[0])
    if len(lIndexes) > 0:
        return lIndexes

    lTemp = oTokenMap.get_token_indexes(lTokens[-1])
    iAdjust = len(lTokens) - 1
    for iIdx, iTemp in enumerate(lTemp):
        iNewIdx = iTemp - iAdjust
        lIndexes.append(iNewIdx)
    return lIndexes
